/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_EXCEPTION_H
#define DI_EXCEPTION_H

#include <exception>
#include <string>

// Original Author Juha Laitinen in HUT/TML.

/** \class Exception
 *
 * Used as base class for all Exceptions. All new Exceptions
 * should be derived from this base class. Use this class in catches
 * to catch all exceptions that have been created by your own code. However, 
 * if possible, use a specialized exception class derived from this class when
 * throwing exceptions.
 */
class DI_Exception : public std::exception
{
public:
  DI_Exception(const char * function,
	       const char * fileline = 0,
	       const char * message = 0,
	       const char * cure = 0,
	       const char * exceptionname = 0) throw();

  //! The default destructor.
  virtual ~DI_Exception() throw();

  const std::string & function() const { return m_function; }
  const std::string & fileline() const { return m_fileline; }
  const std::string & message()  const { return m_message; }
  const std::string & cure()     const { return m_cure; }
  const std::string & exceptionname() const { return m_exceptionname; }

  /// Full description of the problem.
  const std::string & description() const { return m_description; }

protected:
  
  virtual void compileDescription();

  std::string m_function;
  std::string m_fileline;
  std::string m_message;
  std::string m_cure;
  std::string m_exceptionname;

  std::string m_description;
};

/** These exception macros look slightly different on different
 * systems. */

#ifdef __GNUC__ 
#define __DI_FUNCTION__ __PRETTY_FUNCTION__
#else
#define __DI_FUNCTION__ __FUNCTION__
#endif

#define __DI_THROW1(func, file, line, T, msg) \
  throw T(func, file":"__STRING(line), msg);
		     //  throw T(func, file, T, msg);
#define DI_THROW1(T, msg) __DI_THROW1(__DI_FUNCTION__, __FILE__, __LINE__, T, msg)


#endif
