/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_TIME_STAMP_H
#define DI_TIME_STAMP_H

#include <di_types.h>    // ulong

/** Timing info. This class provides two variables. 
    
    Timestamp (stamp()) is the current time stamp.
    
    Hz (hz(), Hertz) is the resolution.
*/

class DI_TimeStamp
{
public:
  DI_TimeStamp() : m_stamp(0), m_hz(44100) {}
  ~DI_TimeStamp() {} 
  
  inline const ulong &stamp() const { return m_stamp; }
  inline void setStamp(ulong s) { m_stamp = s; }
  inline void setStampSecondsD(double s) { m_stamp = (ulong) (m_hz * s); }

  /// Increment the stamp counter.
  inline void incrementStamp(ulong steps) { m_stamp += steps; }

  /// The frequency of sampling
  inline const ulong &hz() const { return m_hz; }
  inline void setHz(ulong freq) { m_hz = freq; }

  /// The number of seconds as double
  inline double secondsD() const { return (double) m_stamp / (double) m_hz; }
  /// The number of whole seconds as unsigned long
  inline ulong secondsUl() const { return m_stamp / m_hz; }

  /// The fractional part of second count
  inline double subSeconds() const 
    { return (double) (m_stamp % m_hz) / (double) m_hz; }
  /// The fractional part of second count as microseconds
  inline ulong subSecondsUs() const 
    { return (ulong) (subSeconds() * 1000000.0); }

protected:

  ulong m_stamp, m_hz;
};

#endif
