/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_VIDEO_AVCODEC_H
#define DI_VIDEO_AVCODEC_H

#include <di_video_input.h>

#include <string>

extern "C" {
  struct AVCodec;
  struct AVCodecContext;
  struct AVFormatContext;
  struct AVFrame;
}

class DI_VideoAvcodecInput : public DI_VideoInput
{
public:
  DI_VideoAvcodecInput();
  ~DI_VideoAvcodecInput();

  virtual const DI_Image * captureImage();

  /// The width of the video stream images.
  virtual int width() const;
  /// The height of the video stream images.
  virtual int height() const;
  /// The frame rate of the video stream.
  virtual float fps() const;
  /// Native image format of the stream.
  virtual DI_ImageFormat imageFormat() const;  
  virtual unsigned int size() const;

  virtual void initialize(DI_ImageFormat fmt = DI_IMAGE_UNKNOWN, 
			  int width = DI_DONT_CARE, 
			  int height = DI_DONT_CARE, 
			  DI_FrameRate framerate = DI_FPS_IGNORE);

  virtual bool isInitialized() const;
  virtual void start();
  virtual bool isStarted() const;
  virtual void stop();
  virtual void finalize();

  void configure(const char * filename, const char * codecname = 0);

  bool close();

  
protected:

  std::string      m_fileName;
  std::string      m_codecName;

  AVCodec        * m_vcodec;
  int              m_vindex; // Video index

  AVCodecContext * m_vcontext;
  AVFrame        * m_frame;

  AVFormatContext *m_ic;
  enum { BUFFER_SIZE = 4096 };

  DI_Image         m_image;
};

#endif
