/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_array2d.h>

#include <mj_error.h>
#include <mj_math.h>
#include <mj_misc.h>

#include <typeinfo>

template <class TElem> 
void MJ_Array2D<TElem>::argCheck(uint row, uint col) const
{
  if((row >= m_rows) || (col >= m_cols))
    MJ_fatal(MJ_ERR_ARGUMENT, typeid(*this).name(), "argCheck");
}

template <class TElem> 
bool MJ_Array2D<TElem>::resize(uint rows, uint cols, bool fatalOnFailure)
{
  uint s = rows * cols;

  if(s == m_rows * m_cols) {
    m_rows = rows;
    m_cols = cols;
    return true;
  }

  TElem *old = m_data;
  TElem *data = 0;

  if(s) {
    data = new TElem[s];
    if(!data) {
      if(fatalOnFailure) MJ_fatal(MJ_ERR_ARGUMENT, "MJ_Array2D::resize");
      return false;
    }
    
    uint mrows = MJ_min(rows, m_rows);
    uint mcols = MJ_min(cols, m_cols);
    
    for(uint i=0; i < mrows; i++)
      for(uint j=0; j < mcols; j++)
	data[i * cols + j] = m_data[i * m_cols + j];
  }

  if(old) delete [] old;
  
  m_data = data;
  m_rows = rows;
  m_cols = cols;
  return true;
}

template <class TElem> 
TElem &MJ_Array2D<TElem>::get(uint row, uint col)
{
  argCheck(row, col);
  return m_data[row * m_cols + col];
}
