/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_ARRAY3D_H
#define MJ_ARRAY3D_H

#include <di_types.h>

#include <assert.h>

template <class T> class MJ_Array3D
{
public:
  MJ_Array3D() 
    : m_layers(0), m_rows(0), m_cols(0), m_elemsPerLayer(0), m_data(0) {}

  MJ_Array3D(const MJ_Array3D & that) 
    : m_layers(0), m_rows(0), m_cols(0), m_elemsPerLayer(0), m_data(0) 
  { *this = that; }

  ~MJ_Array3D(); 
  
  T & get(uint layer, uint row, uint col)
  { return m_data[m_elemsPerLayer * layer + m_cols * row + col]; }

  const T & get(uint layer, uint row, uint col) const
  { return m_data[m_elemsPerLayer * layer + m_cols * row + col]; }

  T & getSafe(uint layer, uint row, uint col)
  { 
    assert(layer < m_layers && row < m_rows && col < m_cols); 
    return m_data[m_elemsPerLayer * layer + m_cols * row + col]; 
  }

  const T & getSafe(uint layer, uint row, uint col) const
  { 
    assert(layer < m_layers && row < m_rows && col < m_cols); 
    return m_data[m_elemsPerLayer * layer + m_cols * row + col]; 
  }

  uint layers() const { return m_layers; }
  uint rows()   const { return m_rows; }
  uint cols()   const { return m_cols; }

  T * data()  { return m_data; }
  T * begin() { return m_data; }
  T * end()   { return m_data + m_layers * m_elemsPerLayer; }

  const T * data() const { return m_data; }
  const T * begin() const { return m_data; }
  const T * end()   const { return m_data + m_layers * m_elemsPerLayer; }

  void setAll(const T & that)
  {
    T * target   = begin();
    T * sentinel = end();
    while(target < sentinel)
      *target++ = that;
  }

  bool resize(uint layers, uint rows, uint cols, 
	      bool fatalOnFailure = true, bool copy = true);
  
  bool clear() { return resize(0, 0, 0); }

  uint elements() const { return m_elemsPerLayer * m_layers; }

  MJ_Array3D & operator = (const MJ_Array3D & that)
  {
    resize(that.m_layers, that.m_rows, that.m_cols);

    T * target   = begin();
    T * sentinel = end();
    const T * source = that.m_data;

    while(target < sentinel)
      *target++ = *source++;

    return *this;
  }

protected:
  uint m_layers, m_rows, m_cols, m_elemsPerLayer;
  
  T * m_data;
};


#endif
