/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_DSP_PLUGIN_LOADER_H
#define MJ_DSP_PLUGIN_LOADER_H

#include <mj_dsp_plugin.h>

#include <qdir.h> 

/** Class that loads the Mustajuuri DSP plugins. */
class MJ_PluginDbeDspLoader
{
public:
  MJ_PluginDbeDspLoader();
  virtual ~MJ_PluginDbeDspLoader();

  /// Load translation for all plugins.
  bool loadTranslations(const QString &paths, 
			const QString &lang);
  /** Load modules from a directory. 

      Recursive if desired. The third argument (depth) should not be
      used by normal users - it used to make shure the file system is
      not travelled too deep. */
  bool loadDir(QDir xDir, 
	       const QString &funcName, 
	       bool bRecursive = false, 
	       int iDepth = 0,
	       MJ_MultiInfoGui *errorGui = 0);

  MJ_PluginDbeList &entryList() { return m_list; }

  const MJ_PluginDbeList
  &entryListConst() const { return m_list; }

  /** Write module menu positions to stream. */
  void writeNames(QTextStream &xStream);

  /** Write hardware module menu positions to stream. */
  void writeHWNames(QTextStream &xStream);

  /** Find an entry with desired identifier. 
   */
  MJ_PluginDbeDsp
    *findEntry(const QString &, MJ_PLuginParameter searchCriterion);

  /** Find an entry with desired menu position. 
   
      @return Non-NULL return value indicates success.
   */
  MJ_PluginDbeDsp *getEntry(ulong i)
  { return (MJ_PluginDbeDsp *) m_list.at(i); }

  bool add(MJ_PluginDbeDsp *entry);
  
  /** Set debugging mode. 
      
      With full debugging extra information is given to the user as
      the class methods execute.
  */
  static void setDebug(bool bDebug) { m_debug = bDebug; }
  
protected:

  void deleteDbe(MJ_PluginDbeBase *);

  
  MJ_PluginDbeList m_list;  
  static bool m_debug;

};

#endif

