/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_FILE_LOAD_INFO_H
#define MJ_FILE_LOAD_INFO_H

#include <qdatetime.h> 
#include <qfileinfo.h> 
#include <qstring.h>

/** Utility for file reload-handling. Useful if you already suspect
    the file that should be read is in the memory. */
class MJ_FileLoadInfo
{
public:
  MJ_FileLoadInfo() { clear(); }
  ~MJ_FileLoadInfo() {}

  /// Returns the name of the file.
  const QString &file() const { return m_file; }
  /// Returns the time when the file was loaded
  const QDateTime &loadTime() const { return  m_loadTime; }

  /** Check if the file needs loading. Returns true if the file is
      newer than load time or the name differs from current file
      name. */
  bool needsLoading(const QString &file);
  bool needsLoading(const QFileInfo &file);
  /// Inform this object that a new file was loaded. 
  void nowLoaded(const QString &file);

  /// Clear contents.
  void clear();

protected:
  QString m_file;
  QDateTime m_loadTime;
};

#endif

