/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_HAS_VALUES_H
#define MJ_HAS_VALUES_H

#include <qdom.h>

#include <map>

class MJ_Value;

class MJ_HasValues
{
public:
  friend class MJ_Value;

  typedef std::map<QString, MJ_Value *> container;
  typedef container::iterator iterator;

  MJ_HasValues();
  virtual ~MJ_HasValues();

  void addValue(const QString & id, MJ_Value *v);
  void forgetValue(const QString & id);
  MJ_Value * value(const QString & id);

  void setTypeName(const QString & name) { m_name = name; }

  virtual QDomElement writeDom(QDomDocument &);
  virtual bool readDom(QDomElement &elem);

  MJ_HasValues * clone() const;

  iterator valuesBegin() { return m_values.begin(); }
  iterator valuesEnd()   { return m_values.end(); }
  
protected:

  MJ_HasValues(const MJ_HasValues &);

  virtual MJ_HasValues * doClone() const;

private:

  MJ_HasValues * m_redirect;

  QString   m_name;

  container m_values;

};

#endif
