/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_IO_CONF_GUI_H
#define MJ_IO_CONF_GUI_H

#include <qwidget.h>

#include <qptrlist.h>

class QGridLayout;

/** A GUI that can be used to change IO configuration. This class is
    used by inheriting it and overriding method appendTypeButtons.

    @see MJ_AudioConf 
 */
class MJ_IoConfGui : public QWidget
{
  Q_OBJECT
public:
  MJ_IoConfGui(QWidget      *parent=0, 
	       const char   *name=0);

  virtual ~MJ_IoConfGui();
  virtual void setEnabled(bool);

  void uncheckAll();
  
  void setChecked(bool input, int index, bool check = true);

public slots:
  void enableInput(int,bool);
  void enableOutput(int,bool);

signals:
  /** Signals that a device is enabled. First bool is true is the
      matter is input, the integer tells the linear index number and
      the final bool indicates whether it is enabled or not. */

  void enableDevice(bool,int,bool);

protected:

  void draw();
  virtual void appendTypeButtons(bool input);
  void enableInOrOut(bool input, int index, bool ena);

  virtual QString ioTypeName(bool input) = 0;
  virtual QString ioName(bool input, int index) = 0;

  /* The full type of all of these is QCheckBox. */
  QList<QWidget> m_inputs;
  QList<QWidget> m_outputs;

  QList<QObject> m_remaps;

  QGridLayout        *m_layout;
};

#endif
