/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_MESSAGE_XML_H
#define MJ_MESSAGE_XML_H

#include <qdom.h> 

class MJ_Message;

class MJ_DomUtil
{
public:
  static void addTextChild(QDomElement &, QDomDocument &doc,
			   const QString &tag,  const QString &value);
  static void addTextChild(QDomElement &, QDomDocument &doc,
			   const QString &tag,  int value);
  static void addTextChild(QDomElement &, QDomDocument &doc,
			   const QString &tag,  double value);

  static double           getDouble(QDomElement &, const QString &tag, 
				    double def=0.0f);
  static int              getInt(QDomElement &, const QString &tag, int def=0);

  static QString          getString(QDomElement &, const QString &tag, 
				    const char * def = 0);

};

/** Reads and writes messages based on XML (DOM) data. */

class MJ_MessageXml : public MJ_DomUtil
{
public:

  static QDomElement writeMsg(QDomDocument &, MJ_Message *);
  static MJ_Message * readMsg(QDomElement &);

  static QDomElement writeMsgBool(QDomDocument &, MJ_Message *);
  static bool readMsgBool(QDomElement &, MJ_Message *);

  static QDomElement writeMsgDouble(QDomDocument &, MJ_Message *);
  static bool readMsgDouble(QDomElement &, MJ_Message *);

  static QDomElement writeMsgInt(QDomDocument &, MJ_Message *);
  static bool readMsgInt(QDomElement &, MJ_Message *);

  static QDomElement writeMsgMidi(QDomDocument &, MJ_Message *);
  static bool readMsgMidi(QDomElement &, MJ_Message *);

  static QDomElement writeMsgMulti(QDomDocument &, MJ_Message *);
  static bool readMsgMulti(QDomElement &, MJ_Message *);

  static QDomElement writeMsgStlString(QDomDocument &, MJ_Message *);
  static bool readMsgStlString(QDomElement &, MJ_Message *);

  static QDomElement writeMsgString(QDomDocument &, MJ_Message *);
  static bool readMsgString(QDomElement &, MJ_Message *);

  static void initMessageXml();

  static void test();

protected:
  static QDomElement writeMsgBasics(QDomDocument &, MJ_Message *);
  static bool readMsgBasics(QDomElement &, MJ_Message *);
};

#endif
