/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MIDI_PROGRAM_TABLE_H
#define MJ_MIDI_PROGRAM_TABLE_H

#include <mj_midi.h>

/** Map MIDI channel number to program number. */
class MJ_MidiProgramTable
{
public:
  MJ_MidiProgramTable() { setLinear(); }
  ~MJ_MidiProgramTable() {}
  
  /** Sets all programs constant value. */
  void setAll(int prog) 
  { for(int i=0; i < MJ_MIDI_CHANNELS; i++) m_map[i] = prog; }

  /** Sets all programs. */
  void setLinear() 
  { for(int i=0; i < MJ_MIDI_CHANNELS; i++) m_map[i] = i; }

  bool setProg(int chan, int prog) 
  { if(chan >= MJ_MIDI_CHANNELS) return false; m_map[chan] = prog; }

  /** Returns the current program. Returns -1 on error. */
  int get(int chan) 
  { if(chan >= MJ_MIDI_CHANNELS) return -1; return m_map[chan]; }

  /** Updates the mappings if necessary. */
  void update(const MJ_MidiEvent *e)
  { if(e->isProgramChange()) setProg(e->channel(), e->program()); }

protected:
  /// Map from MIDI channel number to program number.
  int m_map[MJ_MIDI_CHANNELS];
};

#endif
