/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MODULE_DSP_ARRAY_H
#define MJ_MODULE_DSP_ARRAY_H

#include <mj_module_dsp.h>

extern const char *MJ_ModuleFuncKeys[];

/** Array of modules. This class provides a simple basis for recursive
    DSP modules (eg. modules that contain modules). This class is used
    by inheriting it and adding methods that actually fill the array
    with useful modules. */

class MJ_ModuleDspArray : public MJ_ModuleDsp
{
public:
  MJ_ModuleDspArray();
  virtual ~MJ_ModuleDspArray();

  virtual MJ_ModuleDsp *findModule(const char * id);

  virtual bool dspSetTimer(MJ_TimeStamp *t);
  
  virtual bool dspPrepare();
  virtual bool dspPrepareFast();
  virtual void dspProcessPeriod();
  virtual bool dspStopFast();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);

  virtual void getChildren(QList<MJ_ModuleDsp> &l, bool recursive);

  /** Clear the array without deleting any of the modules. */

  void clearArray(bool alsoDelete = false); 

  void addToArray(MJ_ModuleDsp *, bool append);
  void appendModule(MJ_ModuleDsp *m) { addToArray(m, true); }
  void prependModule(MJ_ModuleDsp *m) { addToArray(m, false); }

  bool removeModule(MJ_ModuleDsp *);

  QCString allocateId();

  MJ_Array<MJ_ModuleDsp *> & moduleArray() { return m_array; }

protected:

  enum MJ_ModuleFunc {
    MJ_MF_PREPARE,
    MJ_MF_PREPARE_FAST,
    MJ_MF_PROCESS_FRAME,
    MJ_MF_STOP_FAST,
    MJ_MF_STOP
  };  
  
  bool dspOperate(MJ_ModuleFunc func, uint start = 0, uint end = 0);

  MJ_Array<MJ_ModuleDsp *> m_array;

  MJ_TimeStamp *m_childTimer;
};

#endif
