/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MODULE_DSP_GUI_DEFAULT_H
#define MJ_MODULE_DSP_GUI_DEFAULT_H

#include <mj_message.h>
#include <mj_message_carrier.h>
#include <mj_module_dsp_gui.h>
#include <mj_parameter.h>

class MJ_HeaderLabel;
class MJ_ParameterString;
class MJ_ParameterBool;

template <class TElem> class MJ_ParameterNumeric;

class QLabel;
class QGridLayout;
class QScrollView;

/// The default stamp-sized GUI.
class MJ_ModuleDspGuiStamp : public MJ_ModuleDspGui
{
  Q_OBJECT
public:
  MJ_ModuleDspGuiStamp(MJ_RtControl *rtparent,
		       MJ_ModuleDsp *module,
		       QWidget      *parent = 0,
		       const char   *name = 0);
  virtual ~MJ_ModuleDspGuiStamp();

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

protected:

  void draw();
  
  MJ_HeaderLabel *m_label;

  static bool m_debug;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// The default postcard-sized GUI.
class MJ_ModuleDspGuiPostCard : public MJ_ModuleDspGui
{
  Q_OBJECT
public:

  MJ_ModuleDspGuiPostCard(MJ_RtControl *rtparent,
			  MJ_ModuleDsp *module,
			  bool          drawThis = true,
			  uint          maxDisplayParameters = 5,
			  MJ_GuiSize    guiSize = MJ_GUI_POSTCARD,
			  QWidget      *parent = 0,
			  const char   *name = 0);
  virtual ~MJ_ModuleDspGuiPostCard();
  
  virtual void moduleModified(const char *);
  
  virtual void setGuiSize(MJ_GuiSize guiSize) { m_guiSize = guiSize; }
    
  static void setDebug(bool debug) { m_debug = debug; }

public slots:

  virtual void setValue(const char *,double);
  virtual void getValue(const char *,double*);

  virtual void setValue(const char *,int);
  virtual void getValue(const char *,int*);

  virtual void setValue(const char *,bool);

  /// Add to undo info.
  virtual void doneValue(const char *);

  virtual void setValueString(const char *, const QString &);

signals:
  void enterNonRealTime(bool);

protected:

  
  virtual void realTimeChange(bool nowRt);

  // void mousePressEvent ( QMouseEvent * me);

  virtual void draw();
  /// Get new parameter values to GUI.
  virtual void update();
  virtual void update2(const char *);

  void addParam(int);

  void addParamDouble(int, MJ_ParameterNumeric<double> *);
  void addParamIntCombo(int, MJ_ParameterNumeric<int> *);
  void addParamIntSlider(int, MJ_ParameterNumeric<int> *);
  void addParamString(int, MJ_ParameterString *);
  void addParamStringCombo(int, MJ_ParameterString *);
  void addParamBool(int, MJ_ParameterBool *);

  /// Sets the module, but does not draw the GUI.
  void setModuleInternal();

  QWidget *childParent();

  QWidget       *m_main;
  QGridLayout   *m_layout;
  QScrollView   *m_scrollView;
  QWidget       *m_scrollChild;

  QCString       m_editParameter;

  MJ_GuiSize     m_guiSize;

  MJ_Parameters  m_parameters;
  uint           m_maxDisplayParameters;
  uint           m_rowsUsed;

  /** For undo control. */
  MJ_MessageArray m_oldParameters;
  MJ_MessageArray m_newParameters;

  static bool    m_debug;
  QList<QObject> m_children;
  // QList<QObject> m_needToDelete;
};

#endif
