/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MODULE_MENUBAR_H
#define MJ_MODULE_MENUBAR_H

#include <mj_message.h>
#include <mj_rt_control.h>
#include <mj_undo_center.h>

#include <qmenubar.h> 

class MJ_DspPLuginInfoGui;
class MJ_ModuleDspGui;
class MJ_SignalFrameGui;

/** Menubar for the normal DSP-objects. */

class MJ_ModuleMenubar : public QMenuBar, public MJ_RtControl
{
  Q_OBJECT
public:
  MJ_ModuleMenubar(MJ_ModuleDspGui * parent, const char * name=0);
  virtual ~MJ_ModuleMenubar();

  inline QPopupMenu *file() { return m_file; }
  inline QPopupMenu *edit() { return m_edit; }

  inline MJ_UndoCenter *undoCenter() { return & m_undoControl; }

public slots:

  void open();
  void save();
  void saveAs();
  void rateConf();

  void setRate(int);
  
  void setFrameSize(int);
  void getFrameSize(int *);

  void newMru();

  void pluginInfo();
  void parameters();

  void pluginInfoDeleted();

protected:
  virtual void realTimeChange(bool nowRt);

  void internalSave(const QString &);

  void addMru(const QString &);

  MJ_ModuleDspGui   *m_dspGui;
  MJ_SignalFrameGui *m_sfg;
  QPopupMenu        *m_file, *m_edit;
  MJ_DspPLuginInfoGui *m_pluginInfo;

  MJ_UndoCenter    m_undoControl;

  int m_fileOpen, m_fileSave, m_fileSaveAs, m_reload;

};

#endif
