/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_PLUGIN_DB_TREE_H
#define MJ_PLUGIN_DB_TREE_H

#include <qstring.h>
#include <qptrlist.h>

#include <mj_plugin_base.h>

class MJ_PluginFilter;
class QTextStream;

/** A generic node in plugin tree. This is an abstract class. */
class MJ_PluginDbTreeNode
{
public:

  MJ_PluginDbTreeNode() {}

  virtual ~MJ_PluginDbTreeNode() {}

  /// Sets the name of the node.
  void setName(const QString &name) { m_name = name; }
  /// Returns the name of the node
  const QString &name() const { return m_name; }

  /// Returns true if the given filter returns true for any child node
  virtual bool filter(MJ_PluginFilter *f) = 0;
  
  /// Prints this node and all child nodes ot screen.
  virtual QTextStream & print(QTextStream &out, 
		      int indentLevel = 0, 
		      bool html = false) = 0;

protected:
///
  QString             m_name;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** A singular plugin tree node. These nodes contain pointers to the
    plugin db.*/
class MJ_PluginDbTreeNodeName : public MJ_PluginDbTreeNode
{
public:
  MJ_PluginDbTreeNodeName() : m_entry(0) {}

  virtual ~MJ_PluginDbTreeNodeName() {}

  /// Set the pointer to the plugin entry
  void setEntry(MJ_PluginDbeBase *entry) { m_entry = entry; }

  /// Get pointer to the plugin entry
  inline MJ_PluginDbeBase *dbe() { return m_entry; }

  QTextStream & print(QTextStream &out, 
		      int indentLevel = 0, 
		      bool html = false);

  bool filter(MJ_PluginFilter *f);
protected:

  MJ_PluginDbeBase *m_entry;
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// A plugin tree node that contain many child nodes.
class MJ_PluginDbTreeNodeMany : public MJ_PluginDbTreeNode
{
public:
  MJ_PluginDbTreeNodeMany() { m_nodes.setAutoDelete(TRUE); }

  virtual ~MJ_PluginDbTreeNodeMany() {}
  
  ulong nofNodes() const;
  
  /// Returns one of the child nodes.
  MJ_PluginDbTreeNode *getNode(int index)
  { return m_nodes.at(index); }
  
  QTextStream & print(QTextStream &out, 
		      int indentLevel = 0, 
		      bool html = false);

  /// Add node to the list of children
  bool addNode(MJ_PluginDbTreeNode *node);

  /// Forms tree from a plugin list (or database)
  bool formTree(QList<MJ_PluginDbeBase> &source, int iDepth = 0);

  bool filter(MJ_PluginFilter *f);
protected:

  QList<MJ_PluginDbTreeNode> m_nodes;
};


#endif
