/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_PLUGIN_FILTER_H
#define MJ_PLUGIN_FILTER_H

class MJ_PluginDbeBase;

/** An abstract class that is used to filter (or select) plugins.

    To use it you need to override the accept method. 
   
    It is often necessary to select only some of the plugins for
    display. Thus the plugins need to be filtered. There is largish
    number of different filtering possibilities. The filtering can be
    based on the capabilities of the plugin, its functionality or its
    metafunctionality. A single all-encompassing filter for would be
    very complex and difficult to use. Instead, One can create and
    combine filtering objects for particular needs.  

    @see MJ_PluginFilterMulti MJ_KernelGui::moduleMenu
*/

class MJ_PluginFilter
{
public:
  MJ_PluginFilter() {}
  virtual ~MJ_PluginFilter() {}

  /** Accept the plugin. This filter returns true if the plugin is
      acceptable. By overriding this method different filtering
      schemes can be constructed. */

  virtual bool accept(MJ_PluginDbeBase *dbe) = 0;
  
};

#endif
