/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_REAL_TIME_SCHEDULING_H
#define MJ_REAL_TIME_SCHEDULING_H

/** Real-time priority levels. The current values reflect the priority
    ranges available in POSIX-compatible systems (1-99).*/
#ifndef WIN32

enum MJ_RtPriority {
  /// Priority for the guarding thread
  MJ_RTP_WATCHDOG = 95,
  /// Priority for audio DSP threads.
  MJ_RTP_AUDIO = 50,
  /// Priority for DSP-related I/O threads.
  MJ_RTP_RT_IO = 30,
  /// Priority for threads that do background file I/O.
  MJ_RTP_FILE_IO = 20,
  /// Priority for threads that are not to be ran with high priority
  MJ_RTP_NOPRIORITY = 0
};

#else

#include <windows.h>
enum MJ_RtPriority {
  MJ_RTP_WATCHDOG = THREAD_PRIORITY_HIGHEST,
  /// Priority for audio DSP threads.
  MJ_RTP_AUDIO = THREAD_PRIORITY_HIGHEST,
  /// Priority for DSP-related I/O threads.
  MJ_RTP_RT_IO = THREAD_PRIORITY_ABOVE_NORMAL,
  /// Priority for threads that do background file I/O.
  MJ_RTP_FILE_IO = THREAD_PRIORITY_NORMAL,
  /// Priority for threads that do background file I/O.
  MJ_RTP_NOPRIORITY = THREAD_PRIORITY_BELOW_NORMAL
};

#endif

bool MJ_realTimeScheduling(int priority, 
			   int pid = -1);

#endif
