/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_STRS_H
#define MJ_STRS_H

/** String container. This class has few methods. Instead it has
    several public members that are character strings. The name of the
    member variables tells the string that can be accessed with it.*/

class MJ_Strs 
{
public:

  static inline bool equals(const char *s1, const char *s2);
  static inline bool equalsFast(const char *s1, const char *s2);
  static inline bool equalsN(const char *s1, const char *s2, unsigned n);
  static inline bool equalsNFast(const char *s1, const char *s2, unsigned n);
  static inline const char *findNext(const char *s1, char c);

  static const char *address;

  static const char *Amplitude;
  static const char *amplitude;

  static const char *Apply;

  static const char *AudioSend;
  static const char *AudioSource;

  static const char *AudioInput;
  static const char *AudioOutput;

  static const char *authorTommi;

  static const char *Attack;
  static const char *attack;

  static const char *azi;

  static const char *Band;

  static const char *Bands;
  static const char *bands;

  static const char *Bandwidth;
  static const char *bandwidth;

  static const char *bits;
  static const char *Bits;

  static const char *Buses;
  static const char *buses;

  static const char *Bypass;
  static const char *bypass;

  static const char *channels;
  static const char *channel;

  static const char *Close;
  static const char *close;

  static const char *del;
  static const char *delay;
  static const char *device;

  static const char *dis;
  static const char *directgain;

  static const char *dspAppendWith;

  static const char *ele;

  static const char *end;

  static const char *feedback;

  static const char *filename;

  static const char *flatten;
  static const char *Flatten;

  static const char *foo;

  static const char *format;

  static const char *framesPerCycle;

  static const char *Frequency;
  static const char *frequency;

  static const char *Gain;
  static const char *gain;

  static const char *inchannells;

  static const char *input;

  static const char *Interval;
  static const char *interval;

  static const char *instrument;

  static const char *Loop;
  static const char *loop;

  static const char *LADSPA_;

  static const char *MIDI;
  static const char *MIDIdevice;
  static const char *MIDIInput;
  static const char *MIDIOutput;

  static const char *mode;

  static const char *MUSTAJUURI_FRAME_SIZE;

  static const char *notecontrol;
  static const char *noteoff;
  static const char *noteoffnow;
  static const char *noteon;

  /** The twelve notes of the standard octave beginning from A. */
  static const char *notes[12];

  static const char *outchannells;

  static const char *output;

  static const char *overwrite;

  static const char *sample;

  static const char *Pan;
  static const char *pan;

  static const char *Parameter;

  static const char *pitch;

  static const char *pluginFactoryFunction;

  static const char *wetratio;

  static const char *rate;

  static const char *Ratio;
  static const char *ratio;

  static const char *sampleRate;

  static const char *Slope;

  static const char *tap;

  static const char *Tempo;
  static const char *tempo;

  static const char *threshold;
  static const char *Threshold;

  static const char *Time;
  static const char *time;

};

/** Test if two strings are equals. One or both strings can be NULL.  */
inline bool MJ_Strs::equals(const char *s1, const char *s2)
{
  if(!s1 || !s2) return s1 == s2;
    
  while(*s1 != '\0' && *s2 != '\0')
    if(*s1++ != *s2++)
      return false;

  return *s1 == *s2;
}

/** Test if two strings are equals. The argument strings must not be
    NULL. */
inline bool MJ_Strs::equalsFast(const char *s1, const char *s2)
{
  while(*s1 != '\0' && *s2 != '\0')
    if(*s1++ != *s2++)
      return false;

  return *s1 == *s2;
}

/** Test if two strings are equals. One or both strings can be NULL.  */
inline bool MJ_Strs::equalsN(const char *s1, const char *s2, unsigned n)
{
  if(!s1 || !s2) return s1 == s2;
    
  for(unsigned i=0; i < n && *s1 != '\0' && *s2 != '\0'; i++) {
    if(*s1++ != *s2++)
      return false;
  }

  return *s1 == *s2;
}

/** Test if two strings are equals. The argument strings must not be
    NULL. */
inline bool MJ_Strs::equalsNFast(const char *s1, const char *s2, unsigned n)
{
  for(unsigned i=0; i < n && *s1 != '\0' && *s2 != '\0'; i++) {
    if(*s1++ != *s2++)
      return false;
  }

  return *s1 == *s2;
}

inline const char *MJ_Strs::findNext(const char *s1, char c)
{
  while(*s1 != '\0') {
    if(*s1 == c) return s1;
    s1++;
  }
  return 0;
}

#endif
