/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_VALUE_HOLD_H
#define MJ_VALUE_HOLD_H

#include <mj_math.h>

/** Anti-jitter filter. This class is used to hide small value
    drifts. */

template <class TElem> 
class MJ_ValueHold
{
public:
  MJ_ValueHold() {}
  /// Sets the value and the threshold.
  MJ_ValueHold(TElem val, TElem minStep) 
    : m_val(val), m_minStep(minStep) {}

  ~MJ_ValueHold() {}

  void set(TElem val, TElem minStep)
  { m_val = val;m_minStep=minStep;}
  
  void putLinear(TElem val)
  {
    if(MJ_abs(val - m_val) > m_minStep)
      m_val = val;
  }

  void putCyclic(TElem val, TElem cycle)
  {
    TElem diff = val - m_val;
    TElem diff2 = diff;
    TElem cycle2 = cycle * 2;

    if(diff >  cycle) diff -= cycle2;
    if(diff < -cycle) diff += cycle2;
    
    if(MJ_abs(diff)>m_minStep || MJ_abs(diff2)>=cycle)
      m_val = val;
  }

  TElem get() const {return m_val;}
  
protected:
  TElem m_val;
  TElem m_minStep;
};

#endif
