/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_VALUE_NUMERIC_H
#define MJ_VALUE_NUMERIC_H

#include <mj_value.h>

template <class T>
class MJ_ValueNumeric : public MJ_Value
{
protected:
  T m_value;
  T m_min;
  T m_max;
public:
  MJ_ValueNumeric(MJ_HasValues * host,
		  const char * id,
		  T val, T min = 0, T max = 0)
    : MJ_Value(host, id),
      m_value(val),
      m_min(min),
      m_max(max)
  {}

  virtual ~MJ_ValueNumeric();

  virtual QDomElement writeDom(QDomDocument &);
  virtual bool readDom(QDomElement &elem);

  inline  T & value() { return m_value; }
  inline  const T & value() const { return m_value; }

  inline operator T & () { return m_value; }
  inline operator const T & () const { return m_value; }

  inline MJ_ValueNumeric & operator = (const T & that)
  { m_value = that; return *this; }
  inline MJ_ValueNumeric & operator = (const MJ_ValueNumeric & that)
  { m_value = that.m_value; return *this; }
};

typedef MJ_ValueNumeric<uint>   MJ_ValueUInt;
typedef MJ_ValueNumeric<int>    MJ_ValueInt;
typedef MJ_ValueNumeric<long>   MJ_ValueLong;
typedef MJ_ValueNumeric<float>  MJ_ValueFloat;
typedef MJ_ValueNumeric<double> MJ_Valueouble;

#endif
