/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_VALUE_T_H
#define MJ_VALUE_T_H

#include <mj_value.h>

template <class T>
class MJ_ValueT : public MJ_Value
{
protected:
  T m_value;

public:
  MJ_ValueT(MJ_HasValues * host,
	    const char * id)
    : MJ_Value(host, id)
  {}

  MJ_ValueT(MJ_HasValues * host,
	    const char * id,
	    T val)
    : MJ_Value(host, id),
      m_value(val)
  {}

  virtual ~MJ_ValueT();

  virtual QDomElement writeDom(QDomDocument &);
  virtual bool readDom(QDomElement &elem);

  inline T & value() { return m_value; }
  inline const T & value() const { return m_value; }

  inline operator T & () { return m_value; }
  inline operator const T & () const { return m_value; }

  inline MJ_ValueT & operator = (const T & that)
  { m_value = that; return *this; }
  inline MJ_ValueT & operator = (const MJ_ValueT & that)
  { m_value = that.m_value; return *this; }
};


typedef MJ_ValueT<QString> MJ_ValueString;
typedef MJ_ValueT<bool>    MJ_ValueBool;

#endif

