/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_VERSION_H
#define MJ_VERSION_H

#include <qstring.h>

class QTextStream;

/** Version number in format x.y.z. In addition to numeric data a
    character string can also be used as descriptor. */
class MJ_Version
{
public:
  static const char *m_label;

  /** The constructor. You can call this method like this:
      
      #include <mj_config.h>
      
      MJ_Version v(MJ_VERSION_MAJOR, MJ_VERSION_MINOR, MJ_VERSION_SUBMINOR);
   */
  MJ_Version(int major = 0, int minor = 0, int subminor=0, QString info = "")
    : m_major(major), m_minor(minor), m_subminor(subminor), m_info(info) {}
  ///
  ~MJ_Version();

  /// Sets the version information
  void set(int major, int minor, int subminor=0, QString info = "")
  { m_major=major; m_minor=minor; m_subminor=subminor; m_info=info.copy(); }

  /// Major version number.
  int vMajor() const { return m_major; }
  /// Minor version number.
  int vMinor() const { return m_minor; }
  /// Subminor version number.
  int vSubminor() const { return m_subminor; }
  /// Any info string.
  const QString & info() const { return m_info; }
  
  /** Returns true if the argument has higher version value. The info
      string is ignored. */
  bool exceedsThis(const MJ_Version &) const;
  /** Compares the numeric index of the version. 
      The info string is ignored. */
  bool equals(const MJ_Version &) const;


  /// Write the version info to QTextStream.
  bool write(QTextStream *s);
  /// Read the version information from a stream.
  bool read(QTextStream *s);

  /** Writes and reads to/from a file. */
  static void test();
protected:
  /// The version numbers.
  int m_major, m_minor, m_subminor;
  /// Info string.
  QString m_info;
};

#endif
