/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_WATCH_DOG_H
#define MJ_WATCH_DOG_H

#include <mj_thread.h>

/** Watch-dog thread that keeps other RT-threads in control. 

    The purpose of this class is to make system hangs very difficult
    to generate.

    This thread works by keeping a pointer to a boolean variable. It
    is the reponsibility of another thread to change the value of the
    variable to true. This thread changes the value of the guarding
    bool to false. If the value remains false after certain time it is
    assumed that The other thread has entered an infinite loop. To
    avoid hanging a system with misbeving RT-threads MJ_WatchDog
    proceeds to kill the whole application. 

    There can only be one watchdog per application.
*/

class MJ_WatchDog : public MJ_Thread
{
public:
  MJ_WatchDog();
  ~MJ_WatchDog();

  /** Set the guarding bool. The guard behaviour can be disabled by
      using NULL pointer as an argument.*/
  void setGuard(bool *guard) { m_guard = guard; }
  void setGuard2(bool *guard) { m_guard2 = guard; }

  /** Request the end of this thread. You need to use waitEnd to make
      sure the thread has indeed finished its work. */
  void requestEnd() { m_loop = false; }

  /** Get the one nice doggie. */
  static MJ_WatchDog *dog() { return  m_dog; }

protected:
  virtual void childLoop();

  bool *m_guard;
  bool *m_guard2;
  bool m_loop;

  static MJ_WatchDog * m_dog;
};

#endif
