/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_ERROR_H
#define MJ_ERROR_H

#include <mj_error_enums.h>

void MJ_failure(MJ_ErrorLevel level,
		MJ_ErrorCode  code, 
		const char *c1, const char *c2 = 0, const char *c3 = 0);

void MJ_warning(MJ_ErrorCode  code, const char *c1, const char *c2 = 0);
void MJ_warning2(MJ_ErrorCode  code, const char *msg, ...);

void MJ_error(MJ_ErrorCode  code, const char *c1, int i);

void MJ_error(MJ_ErrorCode  code, const char *c1);
void MJ_error(MJ_ErrorCode  code, const char *c1, const char *c2);
void MJ_error(MJ_ErrorCode  code, 
		     const char *c1, const char *c2, const char *c3);
void MJ_error2(MJ_ErrorCode  code, const char *msg, ...);
void MJ_error3(bool show, MJ_ErrorCode  code, const char *msg, ...);

void MJ_fatal(MJ_ErrorCode  code, const char *c1, const char *c2 = 0);
void MJ_fatal2(MJ_ErrorCode  code, const char *msg, ...);

void MJ_trace(MJ_ErrorCode  code, const char *c1, const char *c2 = 0);

void MJ_trace(const char *c1, const char *c2 = 0);

void MJ_trace(const char *c1, int i);
void MJ_trace(const char *c1, const char *c2, int i);

void MJ_trace(const char *c1, const void *v);
void MJ_trace2(const char *msg, ...);
void MJ_trace3(bool show, const char *msg, ...);

extern bool MJ_errorsToFile;
extern bool MJ_silenceErrors;

/** Set this function pointer if you want to hook the error messages
    to some other source (in addition to normal output). */

extern void (*MJ_errorHandle) (MJ_ErrorLevel, const char *);

#endif
