/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_ERROR_ENUMS_H
#define MJ_ERROR_ENUMS_H

/// Identical to DI_ErrorLevel
enum MJ_ErrorLevel {
  /// Program execution trace
  MJ_ERRL_TRACE,
  MJ_ERRL_INFORMATION,
  MJ_ERRL_WARNING,
  MJ_ERRL_ERROR,
  MJ_ERRL_FATAL,
  MJ_ERRL_SIZEOF
};

const char *MJ_errorLevelStr(MJ_ErrorLevel);

extern const char *MJ_ErrorLevels[MJ_ERRL_SIZEOF];

/// Error codes.
enum MJ_ErrorCode {
  /// No error
  MJ_ERR_NONE,
  /// Uknown error
  MJ_ERR_UNKNOWN,
  /// Out of memory
  MJ_ERR_MEMORY,
  /// Data corrupted
  MJ_ERR_DATA_CORRUPTED,
  /// Internal error
  MJ_ERR_INTERNAL,
  /// Array size exceeded
  MJ_ERR_ARRAY_EXCEED,
  /// List size exceeded
  MJ_ERR_LIST_EXCEED,
  /// Bad argument
  MJ_ERR_ARGUMENT,
  /// Bad argument
  MJ_ERR_NULL_ARGUMENT,
  /// Resource denied
  MJ_ERR_RC_DENIED,
  /// Only virtual method
  MJ_ERR_VIRTUAL,
  /// Do not call this function
  MJ_ERR_DO_NOT_CALL,
  /// Not implemented yet
  MJ_ERR_UNIMPLEMENTED,
  /// Not initialized yet
  MJ_ERR_UNINITIALIZED,
  /// Real-time performance failure
  MJ_ERR_REALTIME,
  /// Floating point error
  MJ_ERR_FPE,
  /// CPU load too high
  MJ_ERR_CPU_LOAD,
  /// Numeric overflow
  MJ_ERR_NUMERIC_OVERFLOW,
  /// Input problem
  MJ_ERR_INPUT,
  /// System resource unavailable
  MJ_ERR_SYSTEM_RESOURCE,
  /// Buffer underflow
  MJ_ERR_BUFFER_UNDERFLOW,
  /// Buffer overflow
  MJ_ERR_BUFFER_OVERFLOW,
  /// Environment
  MJ_ERR_ENVIRONMENT,
  /// Environment
  MJ_ERR_SYSTEM,
  /// Environment
  MJ_ERR_BAD_STATE,
  /// Do not put any error string
  MJ_ERR_NO_STRING,
  /// Number of error codes
  MJ_ERR_SIZEOF
};

/** Error strings. */
const char *MJ_errorStr(MJ_ErrorLevel);

/// Character strings for error codes:
extern const char *MJ_ErrorCodes[MJ_ERR_SIZEOF];

#endif
