/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_MESSAGE_MULTI_H
#define MJ_MESSAGE_MULTI_H

#include <mj_message.h>

#include <map>
#include <string>

/** A Message that contains other messages. */

class MJ_MessageMulti : public MJ_Message
{
public:
  friend class MJ_MessageQio;

  typedef std::map<std::string, MJ_Message *>::iterator iterator;

  MJ_MessageMulti(long time = -1) : MJ_Message(time), m_autoDelete(true) {}
  
  virtual ~MJ_MessageMulti();

  virtual MJ_MessageType messageType() const;
  virtual MJ_Message *duplicate();
  
  virtual bool write2(DI_DataBuffer2 *b);
  virtual bool read2(DI_DataBuffer2 *b, int readFields);

  void setAutoDelete(bool b) { m_autoDelete = b; }
  bool autoDelete() const { return m_autoDelete; }

  uint childCount() const { return m_children.size(); }
  uint size() const { return m_children.size(); }

  MJ_Message *getChild(const std::string &id);

  double  getDouble(const std::string &id, double def = 0.0);
  int     getInt(const std::string &id, int def = 0);
  std::string getStlString(const std::string &id, const char * def = 0);

  void addChild(const std::string &, MJ_Message *); 

  void clearChildren();
  
  static MJ_Message *createNew();

  MJ_MessageMulti & operator = (const MJ_MessageMulti &);

  iterator begin() { return m_children.begin();}
  iterator end()   { return m_children.end(); }

protected:

  std::map<std::string, MJ_Message *> m_children;

  bool  m_autoDelete;
};

#endif
