/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_MESSAGES_H
#define MJ_MESSAGES_H

#include <mj_message_numeric.h>
#include <di_midi_event.h>

/// STL-string message
class MJ_MessageStlString : public MJ_Message
{
public:

  MJ_MessageStlString(long time = -1) : MJ_Message(time) {}
  MJ_MessageStlString(const MJ_MessageStlString &);
  MJ_MessageStlString(const std::string &);

  virtual ~MJ_MessageStlString() {}

  virtual MJ_MessageType messageType() const;
  virtual MJ_Message *duplicate();

  virtual double toDouble(bool *ok = 0);
  virtual int    toInt(bool *ok = 0);

  inline std::string &string() { return m_string; }
  inline const std::string &stringConst() const { return m_string; }
  inline void setString(const std::string &s) { m_string = s; }

  virtual bool write2(DI_DataBuffer2 *b);
  virtual bool read2(DI_DataBuffer2 *b, int readFields);

  virtual std::string describe();

  static MJ_Message *createNew();
protected:

  std::string m_string;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Boolean message. 

    There are times when things are either wrong or right, true or
    false, red or black, narrow or heavy. Use this message at those
    times.
    
    @see MJ_ParameterBool
*/
class MJ_MessageBool : public MJ_Message
{
public:
  MJ_MessageBool(bool b = false, long time = -1) 
    : MJ_Message(time), m_val(b) { }
  virtual ~MJ_MessageBool() {}

  virtual MJ_MessageType messageType() const;
  virtual MJ_Message *duplicate();

  virtual double toDouble(bool *ok = 0);
  virtual int    toInt(bool *ok = 0);
  virtual bool   toBool(bool *ok = 0);
  
  virtual bool write2(DI_DataBuffer2 *b);
  virtual bool read2(DI_DataBuffer2 *b, int readFields);

  virtual std::string describe();

  inline bool value() const { return m_val; }
  inline void setValue(bool b) { m_val = b; }

  static MJ_Message *createNew();

protected:

  bool m_val;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/// MIDI event message
class MJ_MessageMidiEvent : public MJ_Message
{
public:
  MJ_MessageMidiEvent(long time = -1) : MJ_Message(time) { }
  virtual ~MJ_MessageMidiEvent() {}

  virtual MJ_MessageType messageType() const;
  virtual MJ_Message *duplicate();
  
  virtual bool write2(DI_DataBuffer2 *b);
  virtual bool read2(DI_DataBuffer2 *b, int readFields);

  virtual std::string describe();
  
  DI_MidiEvent &midiMsg() { return m_msg; }

  static MJ_Message *createNew();

protected:
  DI_MidiEvent m_msg;
};

#endif
