/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_FPU_H
#define MJ_FPU_H

/** @file mj_fpu.h 

    This file contains code that is necessary for managing floating
    point unit. Note that not all classes or methods will be available
    on all platfroms.

\code

#include <mj_fpu.h>

#ifdef MJ_HAVE_FPU_MANAGER
  MJ_Fpu::fast();
#endif

\endcode

*/

#include <mj_config.h>

#ifdef __OS_LINUX__

#include <linux/config.h>

#if defined(__x86__) || defined(__x86_64__)

// x86 - Linux code

#include <fpu_control.h>
#include <stdio.h>

#define MJ_HAVE_FPU_MANAGER 1

class MJ_Fpu
{
public:
  static void fast() {

    fpu_control_t c = _FPU_DEFAULT;

    /* fpu_control_t cmask = ~0x10;
       c &= cmask; */
    
    printf("FPU register flags = %x\n", (int) c);

    _FPU_SETCW(c);
  }
};

#else

#error "Unsopported Linux hardware"

#endif  // CONFIG_X86


#elif defined(__OS_IRIX___)


// IRIX/MIPS code

#include <sys/fpu.h>

#define MJ_HAVE_FPU_MANAGER 1

class MJ_Fpu
{
public:
  static void fast() {
    
    union fpc_csr f;

    f.fc_word = get_fpc_csr();
    // f.fc_struct.en_invalid = 1;
    f.fc_struct.en_underflow = 0; // Flush underflows to zero.

    set_fpc_csr(f.fc_word);
    
  }
};

#endif  // Platform switches: __OS_*_


#endif  // MJ_FPU_H
