/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_RING_BUFFER_MULTI_H
#define MJ_RING_BUFFER_MULTI_H

#include <di_types.h>

/** Ring buffer template with multiple samples per frame. */

template <class T>
class MJ_RingBufferMulti
{
public:
  
  MJ_RingBufferMulti() : m_line(0), m_mask(0), m_size(0), m_channels(0) {}
  MJ_RingBufferMulti(uint size, uint channels) 
    : m_line(0), m_mask(0), m_size(0), m_channels(0) { resize(size,channels); }

  bool resize(uint newSize, uint channels);

  uint size() const { return m_size; }
  uint mask() const { return m_mask; }
  uint channels() const { return m_channels; }

  T& getIndex(uint i, uint channel) 
  { return m_line[(i & m_mask) * m_channels + channel]; }
  const T& getIndex(uint i, uint channel) const
  { return m_line[(i & m_mask) * m_channels + channel]; }
  
  T* getFrame(uint i) { return m_line + (i & m_mask) * m_channels; }
  const T* getFrame(uint i) const { return m_line + (i & m_mask)*m_channels; }

  void setAll(const T & v)
  { 
    T * iter = m_line; 
    T * sentinel = iter + (m_size * m_channels);
    while(iter != sentinel) (*iter++) = v;
  }

  MJ_RingBufferMulti<T> &operator = (const MJ_RingBufferMulti &b)
  {
    resize(b.m_size, b.m_channels);
    T * iter = m_line; 
    const T * source = b.m_line;
    T * sentinel = iter + (m_size * m_channels);
    while(iter != sentinel) (*iter++) = (*source++);
    return *this;
  }

protected:
  T      *m_line;
  uint    m_mask;   // Mask for determining index.
  uint    m_size;   // Real size
  uint    m_channels;  // Number of channels
};


#endif

