#import "ParticleForceVortexPatch.h"
#import "vee_vortex.h"

@implementation ParticleForceVortexPatch

- (id)attributes
{
	NSMutableDictionary *at=[[super attributes] mutableCopy];
	[at setObject:@"Kineme Particle Force: Vortex" forKey:@"name"];
	return at;
}


- (id)initWithIdentifier:(id)fp8
{
//	NSLog(@"ParticleForceVortexPatch::initWithIdentifier");
	self=[super initWithIdentifier:fp8];
	if(self)
	{
		[inputNz setDoubleValue:1.0];

		[inputEpsilon setDoubleValue:0.2];
		[inputEpsilon setMinDoubleValue:0.0];
		[inputEpsilon setMaxDoubleValue:1.0];

		[inputOmega setDoubleValue:1.0];
		[inputOmega setMinDoubleValue:0.0];

		[inputRange setDoubleValue:1.0];
		[inputRange setMinDoubleValue:0.0];

		[inputSize setDoubleValue:1.0];

		[inputStrength setDoubleValue:1.0];
		[inputStrength setMinDoubleValue:0.0];
		[inputStrength setMaxDoubleValue:1.0];

		[inputFadeRange setDoubleValue:0.7];
		[inputFadeRange setMinDoubleValue:0.0];
		[inputFadeRange setMaxDoubleValue:1.0];

		[inputMaxForce setDoubleValue:1e10];
		[inputMaxForce setMinDoubleValue:0.0];

		[inputRadialForce setDoubleValue:1.0];
		[inputRadialForce setMinDoubleValue:0.0];

		[inputMinRadius setMinDoubleValue:0.0];

		[inputColor setRed:0.8 green:1.0 blue:0.8 alpha:0.5];
	}
	return self;
}


- (BOOL)setup:(QCOpenGLContext *)context
{
	[super setup:context];

//	NSLog(@"ParticleForceVortexPatch::setup");

	VEE_Vortex *v=new VEE_Vortex();
	[_object setObject:v];

	return YES;
}
- (void)cleanup:(QCOpenGLContext *)context
{
	[super cleanup:context];

//	NSLog(@"ParticleForceVortexPatch::cleanup");
	// _object takes care of releasing our VEE_Vortex
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	[super execute:context time:time arguments:arguments];

//	NSLog(@"ParticleForceVortexPatch::execute");

	VEE_Vortex *v=(VEE_Vortex *)[_object object];

	{
		VEE_Vector3 l=v->getAxes();
		if( [inputNx wasUpdated] )
			l.x=[inputNx doubleValue];
		if( [inputNy wasUpdated] )
			l.y=[inputNy doubleValue];
		if( [inputNz wasUpdated] )
			l.z=[inputNz doubleValue];
		v->setAxes(l);
	}

	if( [inputEpsilon wasUpdated] )
		v->setEpsilon( [inputEpsilon doubleValue] );
	if( [inputTheta wasUpdated] )
		v->setValueFloat("vortexed", [inputTheta doubleValue] * 4.0 );
	if( [inputOmega wasUpdated] )
		v->setMagnitude( [inputOmega doubleValue] * 4.0 );
	if( [inputRange wasUpdated] )
		v->setRange( [inputRange doubleValue] );
	if( [inputSize wasUpdated] )
		v->setHeight( [inputSize doubleValue] );
	if( [inputStrength wasUpdated] )
		v->setValueFloat("strength",[inputStrength doubleValue]);
	if( [inputFadeRange wasUpdated] )
		v->setValueFloat("faderange",[inputFadeRange doubleValue]);
	if( [inputMaxForce wasUpdated] )
		v->setValueFloat("maxforce",[inputMaxForce doubleValue]);
	if( [inputRadialForce wasUpdated] )
		v->setValueFloat("radialforce",[inputRadialForce doubleValue]);
	if( [inputMinRadius wasUpdated] )
		v->setValueFloat("minradius",[inputMinRadius doubleValue]);

	return YES;
}

@end
