#import "ParticleLooksColorTablePatch.h"
#import "ParticleLooksColorTablePatchUI.h"

#import "vee_color_table.h"


float ParticleLooksColorTableDefault[16] =
{
	1.0, 0.8, 1.0, 0.5,
	0.8, 1.0, 1.0, 0.5,
	1.0, 1.0, 0.8, 0.5,
	1.0, 1.0, 1.0, 0.5,
};


@implementation ParticleLooksColorTablePatch

+ (int)executionModeWithIdentifier:(id)fp8
{
	return 0;
}
+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}
+ (int)timeModeWithIdentifier:(id)fp8
{
	return 0;
}
+ (Class)inspectorClassWithIdentifier:(id)fp8
{
	return [ParticleLooksColorTablePatchUI class];
}

- (id)attributes
{
	NSMutableDictionary *at=[[super attributes] mutableCopy];
	[at setObject:@"Kineme Particle Color Table" forKey:@"name"];
	return at;
}


- (id)initWithIdentifier:(id)fp8
{
//	NSLog(@"ParticleLooksColorTablePatch::initWithIdentifier");
	self=[super initWithIdentifier:fp8];
	if(self)
	{
		_rgbatable=new VEE_RgbaTable;
		_colorArrayCount=0;
		[self addColor];
		[self addColor];
		[self addColor];
		[self addColor];
	}
	return self;
}


- (BOOL)setup:(QCOpenGLContext *)context
{
//	NSLog(@"ParticleLooksColorTablePatch::setup");

	_looksColorTable=new VEE_LooksColorTable(_rgbatable);
	_valueLooks=new VEE_ValueLooks;
	_valueLooks->setLooks(_looksColorTable);

	_looks=[[ParticleLooks alloc] initWithLooks:_valueLooks];
	[outputLooks setLooksValue:_looks];

	return YES;
}

- (void)cleanup:(QCOpenGLContext *)context
{
//	NSLog(@"ParticleLooksColorTablePatch::cleanup");

	[_looks release];
	// _valueLooks is freed by _looks
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
//	NSLog(@"ParticleLooksColorTablePatch::execute");
	
	int i;
	for(i=0;i<_colorArrayCount;++i)
	{
		if( [_colorArray[i] wasUpdated] )
		{
			float r,g,b,a;
			[_colorArray[i] getRed:&r green:&g blue:&b alpha:&a];

			_rgbatable->setColor(i,VEE_Rgba(r,g,b,a));


			// it seems we have to reconstruct the VEE_LooksColorTable...
			{
				[_looks release];
				// _valueLooks is freed by _looks
				
				_looksColorTable=new VEE_LooksColorTable(_rgbatable);
				_valueLooks=new VEE_ValueLooks;
				_valueLooks->setLooks(_looksColorTable);

				_looks=[[ParticleLooks alloc] initWithLooks:_valueLooks];
				[outputLooks setLooksValue:_looks];
			}
		}
	}

	return YES;
}


- (void)addColor
{
	{
		_colorArray = (QCColorPort **) realloc
			(
				_colorArray,
				((sizeof(QCColorPort *)) * (_colorArrayCount+1))
			);
				
		_colorArray[_colorArrayCount] =
			[self createInputWithPortClass:[QCColorPort class] forKey:[NSString stringWithFormat:@"Color %i",_colorArrayCount+1] attributes:nil];

		[_colorArray[_colorArrayCount] retain];

		[_colorArray[_colorArrayCount]
			setRed:ParticleLooksColorTableDefault[(_colorArrayCount*4)%16]
			green:ParticleLooksColorTableDefault[(_colorArrayCount*4+1)%16]
			blue:ParticleLooksColorTableDefault[(_colorArrayCount*4+2)%16]
			alpha:ParticleLooksColorTableDefault[(_colorArrayCount*4+3)%16]
		];
	}

	_rgbatable->appendColor2
	(
		ParticleLooksColorTableDefault[(_colorArrayCount*4)%16],
		ParticleLooksColorTableDefault[(_colorArrayCount*4+1)%16],
		ParticleLooksColorTableDefault[(_colorArrayCount*4+2)%16],
		ParticleLooksColorTableDefault[(_colorArrayCount*4+3)%16]
	);

	++_colorArrayCount;
}

- (void)removeColor
{
	if(_colorArrayCount>1)
	{
		{
			_colorArray = (QCColorPort **) realloc
				(
					_colorArray,
					((sizeof(QCColorPort *)) * (_colorArrayCount-1))
				);
			
			[self deleteInputForKey:[NSString stringWithFormat:@"Color %i",_colorArrayCount]];

			[_colorArray[_colorArrayCount-1] release];
		}

		_rgbatable->popColor();
		
		--_colorArrayCount;
	}
}


- (NSDictionary*)state
{
	NSMutableDictionary *stateDict = [[NSMutableDictionary alloc] initWithCapacity:1+_colorArrayCount*4];
	[stateDict addEntriesFromDictionary:[super state]];
	
	[stateDict setObject:[NSNumber numberWithInt:_colorArrayCount] forKey:@"colorArrayCount"];

	int i;
	for(i=0;i<_colorArrayCount;++i)
	{
		float r,g,b,a;
		[_colorArray[i] getRed:&r green:&g blue:&b alpha:&a];
		
		[stateDict setObject:[NSNumber numberWithDouble:r] forKey:[NSString stringWithFormat:@"r%i",i]];
		[stateDict setObject:[NSNumber numberWithDouble:g] forKey:[NSString stringWithFormat:@"g%i",i]];
		[stateDict setObject:[NSNumber numberWithDouble:b] forKey:[NSString stringWithFormat:@"b%i",i]];
		[stateDict setObject:[NSNumber numberWithDouble:a] forKey:[NSString stringWithFormat:@"a%i",i]];
	}

	[stateDict autorelease];
	return stateDict;
}
- (BOOL)setState:(NSDictionary*)state
{
	int i;

	// clean up existing state, if it exists (init creates some)
	for(i=0;i<_colorArrayCount;++i)
	{
		[self deleteInputForKey:[NSString stringWithFormat:@"Color %i",i+1]];
		_rgbatable->popColor();
	}
	
	_colorArrayCount = [[state objectForKey:@"colorArrayCount"] intValue];

	_colorArray = (QCColorPort **) realloc
		(
			_colorArray,
			((sizeof(QCColorPort*)) * (_colorArrayCount))
		);

	for(i=0;i<_colorArrayCount;++i)
	{
		_colorArray[i] =
			[self createInputWithPortClass:[QCColorPort class] forKey:[NSString stringWithFormat:@"Color %i",i+1] attributes:nil];

		float r=[[state objectForKey:[NSString stringWithFormat:@"r%i",i]] doubleValue];
		float g=[[state objectForKey:[NSString stringWithFormat:@"g%i",i]] doubleValue];
		float b=[[state objectForKey:[NSString stringWithFormat:@"b%i",i]] doubleValue];
		float a=[[state objectForKey:[NSString stringWithFormat:@"a%i",i]] doubleValue];

		[_colorArray[i] setRed:	r
						green:	g
						blue:	b
						alpha:	a];

		[_colorArray[i] retain];

		_rgbatable->appendColor2(r,g,b,a);
	}

	return [super setState:state];
}

@end
