#import "ParticleRendererPatch.h"

#import "OpenGL/gl.h"
#import "OpenGL/OpenGL.h"

@implementation ParticleRendererPatch : QCPatch

+ (int)executionModeWithIdentifier:(id)fp8
{
	return 1;
}
+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}
+ (int)timeModeWithIdentifier:(id)fp8
{
	return 1;
}
- (id)attributes
{
	NSMutableDictionary *at=[[super attributes] mutableCopy];
	[at setObject:@"Kineme Particle Renderer" forKey:@"name"];
	return at;
}


- (id)initWithIdentifier:(id)fp8
{
//	NSLog(@"ParticleRendererPatch::initWithIdentifier");
	self=[super initWithIdentifier:fp8];
	if(self)
	{
		[inputSize setDoubleValue:0.5];
		[inputSize setMinDoubleValue:0.0];

//		[inputLengthScale setMinDoubleValue:0.0];

		[inputAlpha setDoubleValue:1.0];
		[inputAlpha setMinDoubleValue:0.0];

//		[inputFOV setDoubleValue:70.0];
//		[inputFOV setMinDoubleValue:0.0];

//		[inputFade setMinDoubleValue:0.0];
//		[inputFixedFade setMinDoubleValue:0.0];

//		[inputTailDispersion setDoubleValue:1.0];

//		[inputContinuousTails setBooleanValue:YES];

		[inputRenderOrder setMaxIndexValue:3];
		[inputBlend setIndexValue: 2];	// additive by default (prettiest)
		[inputZBuffer setIndexValue: 2];	// read-only by default (for proper kineme3d interaction ;)

		//[inputBlending setMaxIndexValue:3];
	}
	return self;
}


- (BOOL)setup:(QCOpenGLContext *)context
{
//	NSLog(@"ParticleRendererPatch::setup");
	CGLContextObj old_ctx = CGLGetCurrentContext();
	CGLSetCurrentContext([context CGLContextObj]);
		veeGl=new VEE_GlRenderer();

//		GEE_GlMaterialBank *m_geeMaterials=new GEE_GlMaterialBank;
//		GEE_GlDisplayLists *m_geeLists=new GEE_GlDisplayLists;
//
//		geeGl=new GEE_GlRenderer();
//			geeGl->setMaterials(m_geeMaterials);
//			geeGl->setDisplayLists(m_geeLists);
//			geeGl->setVeeRenderer(veeGl);
//			geeGl->setShowroupSructure(true);

	_scenebox=NULL;

	_lastFrameTime=0.0;
	CGLSetCurrentContext(old_ctx);

	return YES;
}
- (void)cleanup:(QCOpenGLContext *)context
{
//	NSLog(@"ParticleRendererPatch::cleanup");
	
	delete veeGl;
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	VEE_SceneBox *scenebox=[[inputScene sceneValue] scene];
	if(scenebox!=_scenebox)
	{
//		NSLog(@"ParticleRendererPatch::execute  Setting _scenebox to %08x",scenebox);
		_scenebox=scenebox;
		veeGl->setSceneBox(_scenebox);
	}

	if(_scenebox)
	{
//		NSLog(@"ParticleRendererPatch::execute  Painting.");
		CGLContextObj old_ctx = CGLGetCurrentContext();
		CGLSetCurrentContext([context CGLContextObj]);

		if( [inputSize wasUpdated] )
			veeGl->setValueFloat("size",[inputSize doubleValue]);

//		if( [inputLengthScale wasUpdated] )
//			veeGl->setValueFloat("lengthscale",[inputLengthScale doubleValue]);

		if( [inputAlpha wasUpdated] )
			veeGl->setValueFloat("alphamul",[inputAlpha doubleValue]);

//		if( [inputFOV wasUpdated] )
//			veeGl->setValueFloat("fov",[inputFOV doubleValue]);

//		if( [inputFade wasUpdated] )
//			veeGl->setValueFloat("fade",[inputFade doubleValue]);

//		if( [inputFixedFade wasUpdated] )
//			veeGl->setValueFloat("fixedfade",[inputFixedFade doubleValue]);

//		if( [inputTailDispersion wasUpdated] )
//			veeGl->setValueFloat("taildispersion",[inputTailDispersion doubleValue]);

//		if( [inputContinuousTails wasUpdated] )
//			veeGl->setValueFloat("continuoustails",[inputContinuousTails booleanValue]);

		if( [inputRenderOrder wasUpdated] )
			veeGl->setValueInt("renderorder",[inputRenderOrder indexValue]);

//		if( [inputBlending wasUpdated] )
//			veeGl->setValueInt("blendtype",[inputBlending indexValue]);
		[inputZBuffer setOnOpenGLContext:context];
		[inputCulling setOnOpenGLContext:context];
		[inputBlend setOnOpenGLContext:context];
		

        veeGl->paintGL( [inputShowBoxes booleanValue] ,false,false,time-_lastFrameTime);
		[inputZBuffer unsetOnOpenGLContext:context];
		[inputCulling unsetOnOpenGLContext:context];
		[inputBlend unsetOnOpenGLContext:context];
		
		_lastFrameTime=time;
		CGLSetCurrentContext(old_ctx);
	}

	return YES;
}

@end
