/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_ALGORITHM_H
#define VEE_ALGORITHM_H

/** Miscellanous C++ algorithms to be used with templates. 


    @author Tommi Ilmonen
*/

namespace VEE_Algorithm 
{

  /** Sets all values in range. */
  
  template <class Iterator, class Value> 
  void setAll (Iterator first, Iterator last, const Value & v)
  {
    while(first != last) { (*first) = v; first++; }
  }

  /** Sets all values in range. */
  
  template <class Iterator> 
  void deleteAll (Iterator first, Iterator last)
  {
    while(first != last) { delete *first; first++; }
  }
  
  /** Accumulates all values in the range by v. */
  template <class Iterator, class Value> 
  void accumulate (Iterator first, Iterator last, const Value & v)
  {
    while(first != last) { (*first) += v; first++; }
  }
  
  /** Swaps two values. */
  template <class T> 
  void swap(T & v1, T & v2)
  {
    T tmp = v2;
    v2 = v1;
    v1 = tmp;
  }
  
}

#endif
