/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_BLOW_H
#define VEE_BLOW_H

#include <vee_scene.h>
#include <vee_object.h>
// #include <vee_volume.h>

/**

*/
class VEE_Blow : public VEE_ForceObject
{
public:
  VEE_Blow();
  ~VEE_Blow();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Object * clone() const;

  float magnitude() { return m_magnitude; }

  VEE_Matrix3 *rotation() { return &m_rotation; }

  void setRange(float r) { m_range = r; }
  void setMagnitude(float m) { m_magnitude = m; }
  void setRotation(const VEE_Matrix3 &rot) {
    m_rotation = rot;
    // m_rotationInverse = rot.Inverse();
    m_rotationInverse = rot;
    m_rotationInverse.transpose();
  }
  void setDirection(const VEE_Vector3 &);
  /// Returns the cumulative sum of blow-work over time
  double blown() const { return m_blown; }

  virtual void update(float dt);

  void getForce(VEE_Vector3 *force,
		const VEE_Vector3 &velocity,
                const VEE_Vector3 &location);
  void boundingBoxOfForce(VEE_BoundingBox *box);

  float range() const { return m_range; }

  static VEE_Object * factory();
  
protected:

  VEE_Matrix3 m_rotation;
  VEE_Matrix3 m_rotationInverse;

  float m_range;
  float m_magnitude;
  
  double m_blown;
};

#endif
