/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_damping.h>

#include <vee_math.h>
#include <vee_object.h>
#include <vee_particle.h>
#include <vee_scene.h>
#include <vee_volume_ellipsoid.h>
#include <vee_strs.h>
#include <vee_value.h>


VEE_Damping::VEE_Damping()
  : m_volumeArea(new VEE_VolumeEllipsoid(VEE_Vector3(1000.0, 1000.0, 1000.0))),
    m_magnitude(1.0)
{
  m_ageLimit = 1e+10;
  m_forceType = VEE_FORCE_DRAG;
}

VEE_Damping::VEE_Damping(VEE_Volume *volumeArea, float m, float ageLimit)
  : m_volumeArea(volumeArea),
    m_magnitude(m)
{
  m_ageLimit = ageLimit;
  m_forceType = VEE_FORCE_DRAG;
}

VEE_Damping::~VEE_Damping()
{}

/** Receive and process a value. 

    Parameters:

    <UL>

    <LI> "magnitude", float, range: -1 to 1

    </UL>

*/
bool VEE_Damping::setValue(const char *id, VEE_Value *msg)
{
  if(VEE_Strs::equals(id, VEE_Strs::magnitude)) {
    float v;

    bool ok = msg->getFloat(&v);

    if(ok)
      m_magnitude = v;
    return ok;
  }
  else 
    return VEE_ForceObject::setValue(id, msg);
}

VEE_Object * VEE_Damping::clone() const
{
  return new VEE_Damping(*this);
}

void VEE_Damping::getForce(VEE_Vector3 *force,
			   const VEE_Vector3 &velocity,
			   const VEE_Vector3 &location)
{
  if(m_location == location) {
    *force = VEE_Vector3ZERO;
  }
  else 
  {
    if(m_volumeArea.ptr()->inside(location)) {
      *force = -m_magnitude*velocity;
    }
    else {
      *force = VEE_Vector3ZERO;
    }
  }
}

void VEE_Damping::boundingBoxOfForce(VEE_BoundingBox *box)
{
  m_volumeArea.ptr()->getBoundingBox(box);
  box->move(m_location);
}

VEE_Object * VEE_Damping::factory()
{
  return new VEE_Damping();
}
