/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_DAMPING_H
#define VEE_DAMPING_H

#include <vee_force_object.h>

#include <vee_volume.h>
#include <vee_clonable_pointer.h>

/** An abstract particle that slows other particles down 
    inside the defined area.

    Can be used as friction.
*/

class VEE_Damping : public VEE_ForceObject
{
public:
  VEE_Damping();
  VEE_Damping(VEE_Volume *sourceArea, float m, float ageLimit);
  ~VEE_Damping();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Object * clone() const;

  float magnitude() { return m_magnitude; }

  // VEE_Vector3 *location() { return m_sourceArea->location(); }
  // VEE_Vector3 *centralLocation() { return m_volumeArea->location(); }
  VEE_Volume *volumeArea() { return m_volumeArea.ptr(); }

  virtual void setMagnitude(const float m) { m_magnitude = m; }
  /* virtual void setLocation(const VEE_Vector3 &l)
     { m_volumeArea->setCenter(l[0], l[1], l[2]); }*/
  void setVolume(VEE_Volume *volumeArea) 
  { m_volumeArea = volumeArea; }

  void getForce(VEE_Vector3 *force,
		const VEE_Vector3 &velocity,
                const VEE_Vector3 &location);
  void boundingBoxOfForce(VEE_BoundingBox *box);

  static VEE_Object * factory();

protected:

  VEE_ClonablePointer<VEE_Volume> m_volumeArea;

  float m_magnitude;

};

#endif
