/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_EMITTER_H
#define VEE_EMITTER_H

#include <vee_clonable_pointer.h>
#include <vee_particle2.h>

#include <vector>

class VEE_Emitter : public VEE_Particle2
{
public:
  VEE_Emitter(VEE_Object *type = 0, float creationRate = 100.0f);
  virtual ~VEE_Emitter();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Variable getVariable(const char *) const;

  virtual VEE_Object *clone() const;

  virtual void update(float dt);
  virtual void collision(const VEE_Vector3 &);

  VEE_OperatorList & operators() { return m_plist; }
  void appendOperator(VEE_Operator *o) { m_plist.appendOperator(o); }
  void appendOperator2(VEE_Operator *o, const char * name) 
  { m_plist.appendOperator(o, name); }

  bool removeOperator(const char *name, bool alsoDelete = true);

  void setRate(float hz) { m_interval = 1.0 / hz; }

  // VEE_Emitter & operator = (const VEE_Emitter &);

  void setInheritVelocity(bool inherit) { m_inheritVelocity = inherit; }

  void setType(VEE_Object *o) { m_type = o; }

  VEE_Object * create(float, float);

  void setAccumulate(bool accumulate) { m_accumulate = accumulate; }

  std::vector<VEE_Object *> & accumulated() { return m_justCreated; }

protected:

  float m_interval, m_untilNext;
  float m_rateMul;
  float m_afterCollisionInterval;
  VEE_Vector3 m_lastPos;
  
  bool m_inheritVelocity;
  bool m_accumulate;
  bool m_collisionNewRate;
  bool m_emit,m_emit_first_update_after_on;

  int m_maxCount;
  int m_count;
  

  VEE_OperatorList m_plist;
  VEE_Object      *m_type;
//  VEE_ClonablePointer<VEE_Object> m_type;
  
  std::vector<VEE_Object *> m_justCreated;
};

#endif
