/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_FORCE_PLANE_H
#define VEE_FORCE_PLANE_H

#include <vee_object.h>
#include <vee_scene.h>

/** Plane that pushes particles away.
*/

class VEE_ForcePlane : public VEE_ForceObject
{
public:
  VEE_ForcePlane();
  VEE_ForcePlane(VEE_Vector3 location, VEE_Vector3 velocity,
		 VEE_Vector3 normal, float m);
  ~VEE_ForcePlane();

  virtual bool setValue(const char *, VEE_Value *);  
  virtual VEE_Variable getVariable(const char *) const;
  virtual VEE_Object * clone() const;

  float epsilon() { return m_epsilon; }
  float magnitude() { return m_magnitude; }

  VEE_Vector3 *normal() { return &m_normal; }

  void setEpsilon(const float e) { m_epsilon = e; }
  void setMagnitude(const float m) { m_magnitude = m; }
  void setRange(const float r) { m_range = r; }
  void setRadius(const float r) { m_radius = r; }
  void setNormal(const VEE_Vector3 &n);

  void getForce(VEE_Vector3 *force,
		const VEE_Vector3 &velocity,
                const VEE_Vector3 &location);
  void boundingBoxOfForce(VEE_BoundingBox *box);

  static VEE_Object * factory();

protected:

  VEE_Vector3 m_normal;

  float m_epsilon;
  float m_magnitude;

  // range of the force along plane normal
  float m_range;
  // radius of the plane (circle)
  float m_radius;

private:  
};

#endif
