/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_GRAVITATE_H
#define VEE_GRAVITATE_H

#include <vee_scene.h>
#include <vee_object.h>

/** An abstract particle that pulls other particles towards itself. 
*/

class VEE_Gravitate : public VEE_ForceObject
{
public:
  VEE_Gravitate();

  VEE_Gravitate(VEE_Vector3 location, 
		VEE_Vector3 velocity, 
		float ageLimit,
		float m,
		bool updateVelocity = false);
  
  ~VEE_Gravitate() {}
  
  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Variable getVariable(const char *) const;
  virtual VEE_Object * clone() const;

  float epsilon() { return m_epsilon; }
  float magnitude() { return m_magnitude; }

  void setEpsilon(float e) { m_epsilon = e; }
  void setMagnitude(float m) { m_magnitude = m; }
  void setRange(float r) { m_range = r; }

  /** Sets the decay speed of the force [0 - inf]. */
  void setDecayPower(float decay) { m_decayPower = decay; }

  void getForce(VEE_Vector3 *force,
		const VEE_Vector3 &,
                const VEE_Vector3 &location);

  void boundingBoxOfForce(VEE_BoundingBox *box);

  static VEE_Object * factory();

protected:

  float m_epsilon;
  float m_magnitude;
  float m_decayPower;
  float m_range;
  float m_maxForce;
};

#endif
