/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_gravity.h>

// #include <vee_error.h>
#include <vee_strs.h>
#include <vee_value.h>

VEE_Gravity::VEE_Gravity(VEE_Vector3 g)
  : m_gravity(g)
{
  m_forceType = VEE_FORCE_GRAVITY;
}

/** Receive and process a value. 

    Parameters:

    <UL>
    
    <LI> "gravity", Vector3

    </UL>

*/
bool VEE_Gravity::setValue(const char *id, VEE_Value *msg)
{
  // VEE_trace("VEE_Gravity::setValue # \"%s\"", id);

  if(VEE_Strs::equals(id, "gravity")) {
    VEE_Vector3 v;

    bool ok = msg->getVector3(&v);

    if(ok)
      setGravity(v);
    else {
      float tmp;
      ok = msg->getFloat(&tmp);
      
      if(ok) {
	setGravity(VEE_Vector3(0.0f, tmp, 0.0f));
	// VEE_trace("VEE_Gravity::setValue # Gravity set to %f", tmp);
      }
    }
    return ok;
  }
  else 
    return VEE_ForceObject::setValue(id, msg);
}

VEE_Variable VEE_Gravity::getVariable(const char *id) const
{
  VEE_Variable v;

  if(VEE_Strs::equals(id, "gravity")) {
    v = m_gravity;
  }
  else
    v = VEE_ForceObject::getVariable(id);

  return v;
}

VEE_Object *VEE_Gravity::clone() const
{
  return new VEE_Gravity(*this);
}

void VEE_Gravity::getForce(VEE_Vector3 *force,
			   const VEE_Vector3 &,
			   const VEE_Vector3 &)
{
  *force = m_gravity;
}

void VEE_Gravity::boundingBoxOfForce(VEE_BoundingBox *box)
{
  float v = -1e+12;
  box->low().make(v, v, v);
  box->high() = box->low() * -1.0;
}

VEE_Object * VEE_Gravity::factory()
{
  return new VEE_Gravity;
}
