/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_GRAVITY_H
#define VEE_GRAVITY_H

#include <vee_scene.h>
#include <vee_object.h>

/** Gravity affects other particles with a constant global force
    along a defined vector.
*/
class VEE_Gravity : public VEE_ForceObject
{
public:
  VEE_Gravity(VEE_Vector3 g = VEE_Vector3(0.0, -9.81, 0.0));
  ~VEE_Gravity() {}

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Variable getVariable(const char *) const;

  virtual VEE_Object *clone() const;

  VEE_Vector3 *gravity() { return &m_gravity; }
  void setGravity(const VEE_Vector3 &g) { m_gravity = g; }

  void updateVelocity(float) {}
  void update(float) {}

  void getForce(VEE_Vector3 *force,
		const VEE_Vector3 &,
                const VEE_Vector3 &);
  void boundingBoxOfForce(VEE_BoundingBox *box);

  static VEE_Object * factory();

protected:

  VEE_Vector3 m_gravity;

private:  
};

#endif
