/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_HASH_MAP_H
#define VEE_HASH_MAP_H

#include <map>

template <typename key, typename content, unsigned SIZE = 256>
class VEE_HashMap
{
public:

  typedef std::map<key, content> container;
  typedef typename container::iterator citerator;


  VEE_HashMap() {}
  ~VEE_HashMap() {}

  void clear() { for(unsigned i = 0; i < SIZE; i++) m_hash[i].clear(); }

  inline unsigned keyToHashIndex(key k) { return ((unsigned) k) & (SIZE - 1); }

  inline content & operator[] (key k) { return m_hash[keyToHashIndex(k)][k]; }

  content * find(key k) 
  {
    container & item = m_hash[keyToHashIndex(k)];
    
    citerator it = item.find(k);

    return (it != item.end()) ? & (*it).second : 0;
  }

  void erase(key k)
  {
    container & item = m_hash[keyToHashIndex(k)];
    
    citerator it = item.find(k);
    
    if(it != item.end())
      item.erase(it);
  }

protected:
  
  container m_hash[SIZE];
};

#endif
