/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_HASH_SET_H
#define VEE_HASH_SET_H

#include <set>

template <typename key, unsigned SIZE = 256>
class VEE_HashSet
{
public:

  typedef std::set<key> container;
  typedef typename container::iterator citerator;

  VEE_HashSet() {}
  ~VEE_HashSet() {}

  void clear() { for(unsigned i = 0; i < SIZE; i++) m_hash[i].clear(); }

  inline unsigned keyToHashIndex(key k) { return ((unsigned) k) & (SIZE - 1); }

  void insert(key k) { m_hash[keyToHashIndex(k)].insert(k); }

  bool find(key k) 
  {
    container & item = m_hash[keyToHashIndex(k)];
    
    citerator it = item.find(k);

    return it != item.end();
  }

  void erase(key k)
  {
    container & item = m_hash[keyToHashIndex(k)];
    
    citerator it = item.find(k);
    
    if(it != item.end())
      item.erase(it);
  }

protected:
  
  container m_hash[SIZE];
};

#endif
