/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_MATH_H
#define VEE_MATH_H

template <class T>
inline T VEE_abs(T x1)
{
  return (x1 < 0) ? -x1 : x1;
}

template <class T>
inline T VEE_min(T x1, T x2)
{
  return (x1 < x2) ? x1 : x2;
}

template <class T>
inline T VEE_max(T x1, T x2)
{
  return (x1 > x2) ? x1 : x2;
}

template <class T>
inline T VEE_min(T x1, T x2, T x3)
{
  T x = x1 < x2 ? x1 : x2;
  return x < x3 ? x : x3;
}

template <class T>
inline T VEE_max(T x1, T x2, T x3)
{
  T x = x1 > x2 ? x1 : x2;
  return x > x3 ? x : x3;
}

template <class T>
inline T VEE_clamp(T v, T vmin, T vmax)
{
  if(v < vmin) return vmin;
  if(v > vmax) return vmax;
  return v;
}

template <class T>
inline T VEE_absClamp(T v, T vmin, T vmax)
{
  if(VEE_abs(v) < vmin) return v > (T) 0 ? vmin : -vmin;
  if(VEE_abs(v) > vmax) return v > (T) 0 ? vmax : -vmax;
  return v;
}

template <class T>
inline int VEE_round(T v)
{
  if(v < 0) return (int) (v - 0.5);
  return (int) (v + 0.5);
}

template <class T>
inline int VEE_floor(T v)
{
  if(v < 0) return (int) (v - (T) 0.999999);
  return (int) v;
}

template <class T>
inline int VEE_ceil(T v)
{
  if(v > 0) return (int) (v + (T) 0.999999);
  return (int) v;
}


#define VEE_PI 3.1415926535897931
#define VEE_TWO_PI 6.2831853071795862
#define VEE_HALF_PI 1.57079632679489660
#define VEE_QUARTER_PI 0.78539816339744828

template <class T>
T VEE_deg2rad(T v) { return v / (T) 360 * (T) VEE_TWO_PI; }

template <class T>
T VEE_rad2deg(T v) { return v / (T) VEE_TWO_PI * (T) 360.0; }

template <class T>
T VEE_checkRadianRange(T v) 
{ 
  if(v > (T) VEE_TWO_PI)
    return v - VEE_TWO_PI;
  if(v < (T) -VEE_TWO_PI)
    return v + VEE_TWO_PI;
  
  return v;
}

#endif
