/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_META_BALLS_H
#define VEE_META_BALLS_H

#include <vee_vector3.h>

#include <di_types.h>

#include <vector>

class VEE_BoundingBox;

/** A collection of metaballs for defining an area in 3D. */

class VEE_MetaBalls
{
public:
  VEE_MetaBalls();
  ~VEE_MetaBalls();

  class Ball
  {
  public:

    VEE_Vector3 m_center;
    float       m_radius;

    inline float presence(const VEE_Vector3 &loc, float rfull)
    {
      float radius = (loc - m_center).length();

      if(radius > m_radius)
	return 0.0f;

      if(radius / m_radius < rfull)
	return 1.0f;

      float tmp1 = (m_radius - radius);
      float tmp2 = (m_radius * (1.0 - rfull));
      return tmp1 / tmp2;
    }
  };

  unsigned nofBalls() const { return m_balls.size(); }
  
  Ball & getBall(uint i) { return m_balls[i]; }

  void addBall(const VEE_Vector3 & center, float radius)
  {
    Ball ball;
    ball.m_center = center;
    ball.m_radius = radius;

    m_balls.push_back(ball);
  }

  float presence(const VEE_Vector3 &loc, float rfull);

  void getBoundingBox(VEE_BoundingBox *) const;

  void clear() { m_balls.clear(); }

protected:

  std::vector<Ball> m_balls;
};


#endif
