/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_near_objects.h>

#include <vee_scene.h>

long VEE_NearObjects::objectsInRange
(const VEE_Vector3 &loc, float radius, VEE_SceneBox *box)
{
  m_location = loc;
  m_radiusSquared = radius * radius;

  VEE_Vector3 tmp(radius, radius, radius);

  m_bbox.low()  = m_location - tmp;
  m_bbox.high() = m_location + tmp;
  
  // First we scan a box that contains the bounding box of this search:

  while(! box->boundingBox().contains(m_bbox) && box->parent()) {
    box = box->parent();
  }

  // Now get the number of children:
  
  return objectsInRange2(box);
}

long VEE_NearObjects::objectsInRange2
(VEE_SceneBox *box)
{
  long sum = 0;

  for(VEE_SceneBox::iterator it = box->begin(); it != box->end(); it++) {

    VEE_Vector3 diff = (*it)->location() - m_location;
    float distSquare = diff.lengthSqr();
    if(distSquare < m_radiusSquared)
      sum++;
  }

  // Get the child objects recursively

  box = box->children();

  if(!box)
    return sum;

  for(uint i=0; i < VEE_SB_CHILDREN; i++) {
    if(box->boundingBox().intersects(m_bbox))
      sum += objectsInRange2(box);
    box ++;
  }

  return sum;
}
