/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_OPERATOR_FORCE_GRADIENT_H
#define VEE_OPERATOR_FORCE_GRADIENT_H

#include <vee_object.h>
#include <vee_operator.h>
#include <vee_interpolators.h>

#include <string>

/**  Detects the force gradients in the vicinity of the object and
     chenges some value accordingly. */

class VEE_OperatorForceGradient : public VEE_Operator
{
public:
  VEE_OperatorForceGradient();
  virtual ~VEE_OperatorForceGradient();

  virtual bool setValue(const char *, VEE_Value *);
  virtual void operate(VEE_Object *, float);
  virtual VEE_Operator * clone() const;

  void resize(int curves) { m_curves.resize(curves); }

  void setId(const char *name, int n)
  { expand(n); m_curves[n].m_param = name; }

  void appendPoint(float v, float t, int n)
  { expand(n); m_curves[n].m_curve.appendPoint(v, t); }
  
  void clearCurves() { m_curves.clear(); }

protected:
  
  void expand(uint n) { if(n >= m_curves.size()) m_curves.resize(n+1); }

  class Data {
  public:
    std::string m_param;
    VEE_LinearInterpolation<float> m_curve;
  };

  std::vector<Data> m_curves;
  
  float m_range;
  float m_scale;
  VEE_ForceType m_forceType;
};

#endif
