/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_OPERATOR_RANDOM_TMPL_H
#define VEE_OPERATOR_RANDOM_TMPL_H

#include <vee_operator_random.h>

#include <vee_random.h>

/* Necessary specializations for the specific value types. */

template <>
void VEE_OperatorRandomSelect<float>::deliver(VEE_Object *o, const float & v)
{
  o->setValueFloat(m_id.c_str(), v);  
}

template <>
void VEE_OperatorRandomSelect<int>::deliver(VEE_Object *o, const int & v)
{
  o->setValueInt(m_id.c_str(), v);  
}

template <>
void VEE_OperatorRandomSelect<VEE_Vector3>::deliver
(VEE_Object *o, const VEE_Vector3 & v)
{
  o->setValueVector3(m_id.c_str(), v);  
}

template <>
void VEE_OperatorRandomSelect<VEE_Matrix3>::deliver
(VEE_Object *o, const VEE_Matrix3 & v)
{
  o->setValueMatrix3(m_id.c_str(), v);  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

template <class T>
VEE_OperatorRandomSelect<T>::VEE_OperatorRandomSelect()
{}

template <class T>
VEE_OperatorRandomSelect<T>::~VEE_OperatorRandomSelect()
{}

template <class T>
void VEE_OperatorRandomSelect<T>::operate(VEE_Object *o, float)
{
  uint range = m_values.size();

  if(!range) return;

  float rand = VEE_rand.rand0X(range);
  const T & value = m_values[((uint) rand) % range];
  deliver(o, value);
}

template <class T>
VEE_Operator * VEE_OperatorRandomSelect<T>::clone() const
{
  return new VEE_OperatorRandomSelect(*this);
}

/** Adds a new value to the list of random values. If the value
    already exists, then it is not added at all. */
template <class T>
void VEE_OperatorRandomSelect<T>::addValue(T v)
{ 
  if(find(v) < 0)
    m_values.push_back(v); 
}

template <class T>
bool VEE_OperatorRandomSelect<T>::removeValue(T v)
{
  int i = find(v);
  if(i >= 0) {
    m_values.erase(m_values.begin() + i);
    return true;
  }
  return false;
}

/** Seek the index of a value. If the value is not found, then this
    method returns -1. */

template <class T>
int VEE_OperatorRandomSelect<T>::find(T v)
{
  typename std::vector<T>::iterator it = 
    std::find(m_values.begin(), m_values.end(), v);

  if(it == m_values.end())
    return -1;

  return it - m_values.begin();
}


#endif
