/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_operator_texture_loop.h>

#include <vee_object.h>

VEE_OperatorTextureLoop::VEE_OperatorTextureLoop()
  : m_timeScale(VEE_TS_PERIODIC),
    m_loopTime(1.0),
    m_time(0.0)
{}

VEE_OperatorTextureLoop::~VEE_OperatorTextureLoop()
{}

void VEE_OperatorTextureLoop::operate(VEE_Object *o, float dt)
{
  int size = (*m_textures).size();

  if(!size)
    return;
  
  m_time += dt;
  
  int index;

  if(m_timeScale == VEE_TS_LINEAR) {
    index = (int) (m_time / m_loopTime + 0.5f);
    index = index < size ? index : size - 1;
  }
  else if(m_timeScale == VEE_TS_AGE_MATCH) {
    index = (int) (m_time / o->ageLimit() + 0.5f);
    index = index < size ? index : size - 1;
  }
  else { // VEE_TS_PERIODIC:
    if(m_time > m_loopTime)
      m_time -= m_loopTime;

    index = (int) (m_time / m_loopTime + 0.5f);
    index = index < size ? index : size - 1;
  }

  o->setTexture((*m_textures)[index]);
}

VEE_Operator * VEE_OperatorTextureLoop::clone() const
{
  return new VEE_OperatorTextureLoop(*this);
}

void VEE_OperatorTextureLoop::addTexture(int t)
{ 
  (*m_textures).push_back(t); 
}
