/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_PARAMETERS_H
#define VEE_PARAMETERS_H

#include <vee_parameter.h>

#include <vee_bounding_box.h>

enum VEE_Mapping {
  VEE_MAP_LINEAR = 0,
  VEE_MAP_LOGARITHMIC = 1
};

class VEE_ParameterFloat : public VEE_Parameter
{
public:

  VEE_ParameterFloat() : m_value(0.0), m_min(0.0), m_max(1.0),  
			 m_mapping(VEE_MAP_LINEAR) {}
  VEE_ParameterFloat(float v, float rmin, float rmax, 
		     VEE_Mapping m = VEE_MAP_LINEAR) 
    : m_value(v), m_min(rmin), m_max(rmax),  
      m_mapping(m) {}
  virtual ~VEE_ParameterFloat() {}
  
  virtual VEE_Parameter * clone() const;
  virtual bool setValue(VEE_HasValues *);

  void set(float value, float mi, float ma)
  { m_value = value; m_min = mi; m_max = ma; }

  void setMapping(VEE_Mapping m) { m_mapping = m; }
  VEE_Mapping mapping() const { return m_mapping; }

  void setFloat(float v)
  {
    if(v < m_min) v = m_min;
    if(v > m_max) v = m_max;
    m_value = v;
  }
    
  float value() const { return m_value; }
  float minValue()   const { return m_min; }
  float maxValue()   const { return m_max; }
  
protected:
  float m_value, m_min, m_max;
  VEE_Mapping m_mapping;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ParameterVector3 : public VEE_Parameter
{
public:
  VEE_ParameterVector3();

  virtual ~VEE_ParameterVector3() {}
  
  virtual VEE_Parameter * clone() const;
  virtual bool setValue(VEE_HasValues *);

  void setVector(const VEE_Vector3 &v)
  { m_value = m_bbox.clamp(v); }
  const VEE_Vector3 &vector() const { return m_value; }
  
  VEE_BoundingBox & boundingBox() { return m_bbox; }

  void setBoxLow(const VEE_Vector3 &v) { m_bbox.low() = v; }
  void setBoxHigh(const VEE_Vector3 &v) { m_bbox.high() = v; }
    

protected:
  VEE_Vector3     m_value;
  VEE_BoundingBox m_bbox;
};

#endif
