/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_PARTICLE_H
#define VEE_PARTICLE_H

#include <vee_object.h>
#include <vee_looks.h>

#include <string>

/** Particle base class. */

class VEE_Particle : public VEE_Object
{
public:

  VEE_Particle() : m_looks(0), m_visible(true) 
  { m_objectType = VEE_OBJECT_PARTICLE; }

  VEE_Particle(const VEE_Particle &);
  VEE_Particle(VEE_Vector3 location, VEE_Vector3 velocity,
	       float ageLimit, VEE_Looks *l);
  virtual ~VEE_Particle();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Variable getVariable(const char *) const;

  virtual VEE_Object *clone() const;

  /// Update the location of this object.
  virtual void update(float);
  virtual void render(VEE_Renderer *);

  void setLooks(VEE_Looks *l);
  VEE_Looks *looks() { return m_looks; }

  void setVisible(bool visible) { m_visible = visible; }
  bool visible() const { return m_visible; }

  static VEE_Object * factory();

protected:

  VEE_Looks *m_looks;
  bool       m_visible;
};

#endif
