/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_rand2d.h>

#include <vee_error.h>
#include <vee_object.h>
#include <vee_random.h>
#include <vee_values.h>

VEE_RandOnMesh2D::VEE_RandOnMesh2D()
  : m_decay(0.9),
    m_threshold(5.0)
{
  m_align.identity();
  m_stretch.make(1.0);
}

VEE_RandOnMesh2D::~VEE_RandOnMesh2D()
{}

bool VEE_RandOnMesh2D::setValue(const char *id, VEE_Value *v)
{
  bool ok = false;

  if(strcmp(id, "decay") == 0) {
    float tmp;

    ok = v->getFloat(&tmp);

    if(ok)
      m_decay = tmp;
  }
  else if(strcmp(id, "threshold") == 0) {
    float tmp;

    ok = v->getFloat(&tmp);

    if(ok)
      m_threshold = tmp;
  }
  else if(strcmp(id, "stretch") == 0) {
    VEE_Vector3 tmp;

    ok = v->getVector3(&tmp);

    VEE_trace("VEE_RandOnMesh2D::setValue # %s %d [%f %f %f]", id, (int) ok,
	      tmp[0], tmp[1], tmp[2]);

    if(ok)
      m_stretch = tmp;
  }
  else if(strcmp(id, "align") == 0) {
    VEE_Matrix3 tmp;

    ok = v->getMatrix3(&tmp);

    if(ok)
      m_align = tmp;
  }
  else
    ok = false;

  return ok;
}

void VEE_RandOnMesh2D::operate(VEE_Object *o, float)
{
  VEE_Vector3 v1 = newVector();

  VEE_ValueVector3 vv(v1);

  o->setValue(m_id.c_str(), &vv);
}

VEE_Operator * VEE_RandOnMesh2D::clone() const
{
  return new VEE_RandOnMesh2D(*this);
}

VEE_Vector3 VEE_RandOnMesh2D::newVector()
{
  // VEE_trace("VEE_RandOnMesh2D::newVector");

  { 
    // Attenuate the histogram values
    float * f = m_histogram.begin();
    float * fe = m_histogram.end();
    
    while(f < fe)
      (*f++) *= m_decay;
  }

  VEE_Vector3 v;

  float cols = (float) m_histogram.cols();
  float rows = (float) m_histogram.rows();

  int irow = 0;
  int icol = 0;
  int rounds;

  int roundsMax = m_histogram.elements();

  for(rounds = roundsMax; rounds > 0; rounds--) {
    
    float x = VEE_rand.rand01();
    float y = VEE_rand.rand01();
    
    icol = (int) (x * cols);
    irow = (int) (y * rows);
    
    float * tmp = & m_histogram.getFast(irow, icol);
    if(*tmp < m_threshold) {
      
      /* VEE_trace("VEE_RandOnMesh2D::newVector # Hit [%d %d] after %d", 
	 irow, icol, roundsMax - rounds); */
      
      v.make(x - 0.5f, y - 0.5f, 0.0f);
      rounds = 0;
      (*tmp) += 1.0;
    }
  }


  v.scale(m_stretch);

  return m_align * v;
}
