/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_rate_control.h>

#include <vee_error.h>
#include <vee_near_objects.h>
#include <vee_object.h>
#include <vee_strs.h>

VEE_RateControl::VEE_RateControl()
{
  defaults();
}

VEE_RateControl::~VEE_RateControl()
{}

void VEE_RateControl::defaults()
{
  m_param = VEE_Strs::rate;

  m_rateIntegral = 0.0;
  m_dieAfter = 1000.0;
  m_rateToHeat = 50.0;
  m_currentRate = 0.0;
  m_maxRate = 100.0;
  m_maxHeat = 1000.0;
  m_requireHeat = 300.0;
  m_heatCoeff = 0.05;
  m_heat = 0.0;
  m_radius = 1.0;
}


void VEE_RateControl::operate(VEE_Object *o, float dt)
{
  // VEE_trace("VEE_RateControl::operate %p %p %p", this, o, o->sceneBox());

  // VEE_trace("VEE_RateControl::operate # ENTER");

  if(m_rateIntegral > m_dieAfter)
    return;

  VEE_NearObjects objCounter;

  long count = 
    objCounter.objectsInRange(o->location(), m_radius, o->sceneBox());

  float effect = dt * m_heatCoeff;

  float add = count * effect * m_rateToHeat;
  m_heat = m_heat * (1.0f - effect) + add;
  
  if(m_heat > m_maxHeat) m_heat = m_maxHeat;

  if(m_heat < m_requireHeat)
    m_currentRate = 0;
  else {
    m_currentRate = m_heat / m_maxHeat * m_maxRate;
  }

  // VEE_trace("VEE_RateControl::operate # %3.f, %.3f", m_heat, m_currentRate);

  // VEE_trace("VEE_RateControl::operate # 1 %p %f", this, m_currentRate);

  o->setValueFloat(m_param.c_str(), m_currentRate);

  // VEE_trace("VEE_RateControl::operate # 2 %p %f", this, m_currentRate);

  m_rateIntegral += m_currentRate * dt;
  
  if(m_rateIntegral >= m_dieAfter)
    o->setValueFloat(m_param.c_str(), 0.0);

}

VEE_Operator * VEE_RateControl::clone() const
{
  return new VEE_RateControl(*this);
}
