/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_REFERENCE_OBJECT_H
#define VEE_REFERENCE_OBJECT_H

/// Utility for VEE_ReferenceObject
template <typename T>
class VEE_ReferenceOjb
{
public:
  VEE_ReferenceOjb() : m_count(1) {}

  T m_object;
  unsigned m_count;
};

/// Smart pointer
template <typename T>
class VEE_ReferenceObject
{
public:
  VEE_ReferenceObject() : m_holder(new VEE_ReferenceOjb<T>) {}
  VEE_ReferenceObject(const T &obj) 
    : m_holder(new VEE_ReferenceOjb<T>) { m_holder->m_object = obj; }
  
  VEE_ReferenceObject(const VEE_ReferenceObject &that)
  {
    m_holder = ((VEE_ReferenceObject *) & that)->m_holder; 
    m_holder->m_count++;
  }
  
  ~VEE_ReferenceObject() { breakLink(); }

  T & ref() { return m_holder->m_object; }
  const T & ref() const { return m_holder->m_object; }

  /// Create a deep copy of the object
  void deepCopy(const VEE_ReferenceObject &that)
  { 
    if(m_holder->count > 1) {
      --m_holder->m_count;
      m_holder = new VEE_ReferenceOjb<T>;
    }    
    m_holder->m_object = that.m_holder->m_object;
  }

  T & operator *() { return m_holder->m_object; }
  const T & operator *() const { return m_holder->m_object; }

  /// Link to the source object
  VEE_ReferenceObject & operator = (const VEE_ReferenceObject &that)
  { 
    if(m_holder == that.m_holder) return *this;

    breakLink(); 
    m_holder = ((VEE_ReferenceObject *) & that)->m_holder; 
    m_holder->m_count++;

    return *this;
  }

protected:

  void breakLink()
  { if(! --m_holder->m_count) delete m_holder; }
  
  VEE_ReferenceOjb<T> *m_holder;
};

#endif

