/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_RENDERER_H
#define VEE_RENDERER_H

#include <vee_has_values.h>

class VEE_ForceObject;
class VEE_Particle;
class VEE_Polygon;

/** Available blending modes. */

enum VEE_Blending {
  /// Add the foreground to the background
  VEE_BLEND_NEW,
  /// Add the foreground to the background
  VEE_BLEND_ADDITIVE,
  /// Subtract the foreground from the background
  VEE_BLEND_SUBTRACTIVE,
  /// Add the foreground and fade the background
  VEE_BLEND_USUAL
};

/** Abstract base class for all renderers that can draw VEE scenes. */

class VEE_Renderer : public VEE_HasValues
{
public:
  VEE_Renderer() {}
  virtual ~VEE_Renderer() {}

  virtual void initRenderState() = 0;
  virtual void closeRenderState() = 0;

  virtual void renderParticle(VEE_Particle *) = 0;
  virtual void renderPolygon(VEE_Polygon *) = 0;
  virtual void renderForceObject(VEE_ForceObject *) = 0;
};

#endif
