/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_VALUE_H
#define VEE_VALUE_H

#include <vee_matrix3.h>
#include <vee_vector3.h>

#include <vee_has_values.h>

/** Base class for values. Values are typically used to deliver
    data to VEE_Object -sub-classes. 

    This class contains virtual functions that that should be
    overridden in value types that can provide that kind of
    output. The default implementations of the value types simply
    return "false". The overriding implementations should return
    "true".
*/
class VEE_Value : public VEE_HasValues
{
public:
  VEE_Value() {}
  virtual ~VEE_Value() {}

  virtual VEE_Value * clone() const;

  virtual bool getFloat(float *);
  virtual bool getInt(int *);
  virtual bool getVector3(VEE_Vector3 *);
  virtual bool getMatrix3(VEE_Matrix3 *);

  bool getFloat2(bool &, const char *c1, const char *c2, float *);
  bool getInt2(bool &, const char *c1, const char *c2, int *);
};



#endif
