/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_value_operator.h>

#include <vee_object.h>
#include <vee_values.h>

VEE_ValueOperator::VEE_ValueOperator()
  : m_value(0)
{}

VEE_ValueOperator::VEE_ValueOperator(const std::string &id, VEE_Value *v)
  : m_value(v), 
    m_id(id)
{}

VEE_ValueOperator::VEE_ValueOperator(const std::string &id, float v)
  : m_value(new VEE_ValueFloat(v)),
    m_id(id)
{}

VEE_ValueOperator::VEE_ValueOperator(const std::string &id, const VEE_Matrix3 &m)
  : m_value(new VEE_ValueMatrix3(m)),
    m_id(id)
{}

VEE_ValueOperator::VEE_ValueOperator(const std::string &id, const VEE_Vector3 &m)
  : m_value(new VEE_ValueVector3(m)),
    m_id(id)
{}

VEE_ValueOperator::VEE_ValueOperator(const VEE_ValueOperator &that)
  : m_value(that.m_value ? that.m_value->clone() : 0),
    m_id(that.m_id)
{}

VEE_ValueOperator::~VEE_ValueOperator()
{
  delete m_value;
}

bool VEE_ValueOperator::setValue(const char *id, VEE_Value *v)
{
  if(m_value) return m_value->setValue(id, v);

  /* VEE_ValueFloat *vf = dynamic_cast<VEE_ValueFloat *> (m_value);

  if(vf) {

    if(m_id != id && m_id.size())
      return false;
    
    float tmp;
    bool ok = v->getFloat(&tmp);
    if(ok) vf->setFloat(tmp);
    return ok;
  }

  VEE_ValueVector3 *vv3 = dynamic_cast<VEE_ValueVector3 *> (m_value);

  if(vv3) {

    if(m_id != id && m_id.size())
      return false;
    
    VEE_Vector3 tmp;
    bool ok = v->getVector3(&tmp);
    if(ok) vv3->setValue(tmp);
    return ok;
    }*/

  return false;
}

/** Sets the value with no interpolation etc. */
void VEE_ValueOperator::operate(VEE_Object *o, float)
{
  if(m_value)
    o->setValue(m_id.c_str(), m_value);
}

VEE_Operator * VEE_ValueOperator::clone() const
{
  return new VEE_ValueOperator(*this);
}

void VEE_ValueOperator::set(const std::string &id, VEE_Value *v)
{
  if(m_value)
  delete m_value;

  m_value = v;
  m_id = id;
}

VEE_ValueOperator & VEE_ValueOperator::operator = (const VEE_ValueOperator &that)
{
  delete m_value;

  m_value = that.m_value ? that.m_value->clone() : 0;
  m_id = that.m_id;

  return * this;
}
