
/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_value_random.h>

#include <vee_math.h>
#include <vee_random.h>
#include <vee_strs.h>

VEE_ValueRandomVector3::VEE_ValueRandomVector3()
  : m_randMin(-1.0, -1.0, -1.0),
    m_randMax(1.0, 1.0, 1.0)
{
  m_randRotation.identity();
}

VEE_ValueRandomVector3::~VEE_ValueRandomVector3()
{}
  
VEE_Value * VEE_ValueRandomVector3::clone() const
{
  return new VEE_ValueRandomVector3(*this);
}

bool VEE_ValueRandomVector3::setValue(const char *id, VEE_Value *v)
{
  VEE_Vector3 tmp;
  bool ok = v->getVector3(&tmp);

  if(!ok) return false;

  if(VEE_Strs::equals(id, "min"))
    m_randMin = tmp;
  else if(VEE_Strs::equals(id, "max"))
    m_randMin = tmp;
  else
    return false;
  
  return true;
}

bool VEE_ValueRandomVector3::getVector3(VEE_Vector3 *v)
{
  VEE_Vector3 tmp;

  for(int i=0; i < 3; i++)
    tmp[i] = m_randMin[i] + VEE_rand.rand0X(m_randMax[i] - m_randMin[i]);
  
  *v = m_randRotation * tmp;

  return true;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

VEE_ValueRandomRotation::VEE_ValueRandomRotation()
  : m_axes(0.0f, 1.0f, 0.0f),
    m_maxRotation(VEE_TWO_PI)
{
}

VEE_ValueRandomRotation::~VEE_ValueRandomRotation()
{}

VEE_Value * VEE_ValueRandomRotation::clone() const
{
  return new VEE_ValueRandomRotation(*this);
}

bool VEE_ValueRandomRotation::setValue(const char *id, VEE_Value *v)
{
  bool ok = false;

  if(VEE_Strs::equals(VEE_Strs::axes, id)) {
    VEE_Vector3 tmp;
    ok = v->getVector3(&tmp);
    
    if(ok)
      m_axes = tmp;
  }
  else if(VEE_Strs::equals("max", id)) {
    float tmp;
    ok = v->getFloat(&tmp);
    
    if(ok)
      m_maxRotation = VEE_clamp(tmp, 0.0f, (float) VEE_TWO_PI);
  }

  return ok;
}
 
bool VEE_ValueRandomRotation::getMatrix3(VEE_Matrix3 *m)
{
  m->rotateAroundAxis(m_axes, VEE_rand.rand0X(m_maxRotation));

  return true;
}
  
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////



VEE_ValueRandomFloat::VEE_ValueRandomFloat(float fmin, float fmax)
  :  m_min(fmin),
     m_max(fmax),
     m_range(fmax - fmin)
{}

VEE_ValueRandomFloat::~VEE_ValueRandomFloat()
{}
  
VEE_Value * VEE_ValueRandomFloat::clone() const
{
  return new VEE_ValueRandomFloat(*this);
}

bool VEE_ValueRandomFloat::setValue(const char *id, VEE_Value *v)
{
  float tmp;
  bool ok = v->getFloat(&tmp);
  
  if(!ok) return false;

  if(VEE_Strs::equals(id, "min"))
    m_min = tmp;
  else if(VEE_Strs::equals(id, "max"))
    set(m_min, tmp);
  else if(VEE_Strs::equals(id, VEE_Strs::range)) {
    m_range = tmp;
    m_max = m_min + tmp;
  }
  else 
    return false;

  return true;
}

bool VEE_ValueRandomFloat::getFloat(float *v)
{
  *v = m_min + VEE_rand.rand0X(m_range);
  return true;
}

bool VEE_ValueRandomFloat::getInt(int *v)
{
  *v = (int) (m_min + VEE_rand.rand0X(m_range));
  return true;
}
