/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_value_str.h>

VEE_ValueString::VEE_ValueString()
{}

VEE_ValueString::~VEE_ValueString()
{}

VEE_Value * VEE_ValueString::clone() const
{
  return new VEE_ValueString(*this);
}

bool VEE_ValueString::getFloat(float *x)
{
  if(!m_string.size())
    return false;

  const char * str = m_string.c_str();
  char * str2 = (char *) str;

  *x = strtod(str, & str2);

  return str2 > str;
}

bool VEE_ValueString::getInt(int *x)
{
  if(!m_string.size())
    return false;

  const char * str = m_string.c_str();
  char * str2 = (char *) str;

  *x = strtol(str, & str2, 10);

  return str2 > str;
}

bool VEE_ValueString::getVector3(VEE_Vector3 *x)
{
  return sscanf(m_string.c_str(), "%f %f %f", & x->x, & x->y, & x->z) == 3;
}

bool VEE_ValueString::getMatrix3(VEE_Matrix3 *m)
{
  return sscanf(m_string.c_str(), "%f %f %f %f %f %f %f %f %f", 
		& m->get(0, 0), & m->get(0, 1), & m->get(0, 2), 
		& m->get(1, 0), & m->get(1, 1), & m->get(1, 2), 
		& m->get(2, 0), & m->get(2, 1), & m->get(2, 2)) == 9;
}
