/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */



#include <vee_volume_box.h>

#include <vee_bounding_box.h>
#include <vee_random.h>
#include <vee_misc.h>

#include <sys/types.h>
#include <math.h>

VEE_VolumeBox::VEE_VolumeBox()
  : m_axes(1.0f, 1.0f, 1.0f)
    // m_rotation(VEE_Matrix3IDENTITY)
{
  m_rotation.identity();
  m_random = new VEE_RandomUniform;
}

VEE_VolumeBox::~VEE_VolumeBox()
{}

void VEE_VolumeBox::setAxes(float a1, float a2, float a3)
{
  m_axes[0] = a1;
  m_axes[1] = a2;
  m_axes[2] = a3;
}

VEE_Vector3 VEE_VolumeBox::newRandomPoint()
{
  VEE_Vector3 randomVect;
  
  for(int i=0; i < 3; i++)
    randomVect[i] = m_random.ptr()->rand()*m_axes[i];
  
  return (m_rotation * randomVect);
}

VEE_Volume * VEE_VolumeBox::clone() const
{
  return new VEE_VolumeBox(*this);
}

void VEE_VolumeBox::getBoundingBox(VEE_BoundingBox *box)
{
  float max = m_axes[0];

  if(max < m_axes[1]) max = m_axes[1];
  if(max < m_axes[2]) max = m_axes[2];
  max = 1.74*max;

  box->low()  = VEE_Vector3(-max,-max,-max);
  box->high() = VEE_Vector3(max,max,max);
/*
  (*low)[0] = m_center[0]-max;
  (*low)[1] = m_center[1]-max;
  (*low)[2] = m_center[2]-max;
  (*high)[0] = m_center[0]+max;
  (*high)[1] = m_center[1]+max;
  (*high)[2] = m_center[2]+max;
*/
}

float VEE_VolumeBox::inside(const VEE_Vector3 &location)
{
  return VEE_within(location, -m_axes, m_axes);
}
