/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_VOLUME_DISK_H
#define VEE_VOLUME_DISK_H

#include <vee_volume.h>

#include <vee_matrix3.h> 

class VEE_Random;

/** Disk-shaped object volume.
*/
class VEE_VolumeDisk : public VEE_Volume
{
public:
  VEE_VolumeDisk();
  virtual ~VEE_VolumeDisk();

  virtual VEE_Vector3 newRandomPoint();
  virtual VEE_Volume * clone() const;
  virtual float inside(const VEE_Vector3 &)
    { return 0.0; }
  virtual void getBoundingBox(VEE_BoundingBox *);

  float radius() { return m_radius; }
  VEE_Matrix3 *rotation() { return &m_rotation; }
  void setCenter(float x, float y, float z);

  void setRadius(float r) { m_radius = r; }
  void setRotation(VEE_Matrix3 &rot) { m_rotation = rot; }
  void setRandom(VEE_Random *r) { m_random = r; }


protected:

  VEE_Matrix3  m_rotation; 
  float  m_radius;

  VEE_Random   *m_random; 
};


#endif
