/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_volume_ellipsoid.h>

#include <vee_bounding_box.h>
#include <vee_random.h>

#include <sys/types.h>
#include <math.h>

VEE_VolumeEllipsoid::VEE_VolumeEllipsoid()
  : m_axes(1.0f, 1.0f, 1.0f)
    // m_rotation(VEE_Matrix3IDENTITY)
{
  m_rotation.identity();
  m_random = new VEE_RandomUniform;
}

VEE_VolumeEllipsoid::VEE_VolumeEllipsoid(const VEE_Vector3 & axes)
  : m_axes(axes),
    m_rotation(VEE_Matrix3IDENTITY)
{
  m_random = new VEE_RandomUniform;
}

VEE_VolumeEllipsoid::~VEE_VolumeEllipsoid()
{}

void VEE_VolumeEllipsoid::setAxes(float a1, float a2, float a3)
{
  m_axes[0] = a1;
  m_axes[1] = a2;
  m_axes[2] = a3;
}

VEE_Vector3 VEE_VolumeEllipsoid::newRandomPoint()
{
  VEE_Vector3 randomVect;

  for(int i=0; i < 3; i++)
    randomVect[i] = m_random.ptr()->rand();

  randomVect.normalize();

  for(int i=0; i < 3; i++)
    randomVect[i] *= m_axes[i];

  randomVect *= m_random.ptr()->rand();

  return (m_rotation * randomVect);
}

void VEE_VolumeEllipsoid::getBoundingBox(VEE_BoundingBox *box)
{
  float max = m_axes[0];
  if(max < m_axes[1]) max = m_axes[1];
  if(max < m_axes[2]) max = m_axes[2];

  VEE_Vector3 b(max,max,max);

  box->low()  = - b;
  box->high() =   b;
}

VEE_Volume * VEE_VolumeEllipsoid::clone() const
{
  return new VEE_VolumeEllipsoid(*this);
}

float VEE_VolumeEllipsoid::inside(const VEE_Vector3 &location)
{
  VEE_Vector3 p = m_rotation * location;
  float f=0, t;

  for(uint i=0; i<3; i++) {
    t = p[i]/m_axes[i];
    f += t*t;
  }

  return (f < 1) ? 1.0 : 0.0;
}
