/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_VOLUME_ELLIPSOID_H
#define VEE_VOLUME_ELLIPSOID_H

#include <vee_volume.h>

#include <vee_clonable_pointer.h> 
#include <vee_matrix3.h> 
#include <vee_random.h> 



/** Ellipsoid-shaped object volume. 

    This class can also be used to create ball-shaped areas if you set
    the matrices properly.
*/
class VEE_VolumeEllipsoid : public VEE_Volume
{
public:
  VEE_VolumeEllipsoid();
  VEE_VolumeEllipsoid(const VEE_Vector3 & axes);
  virtual ~VEE_VolumeEllipsoid();

  virtual VEE_Vector3 newRandomPoint();
  virtual VEE_Volume * clone() const;

  virtual void getBoundingBox(VEE_BoundingBox *);

  virtual float inside(const VEE_Vector3 &location);

  /* VEE_Vector3 *centralLocation() { return &m_center; }
     VEE_Vector3 *location() { return &m_center; }*/
  VEE_Vector3 *axes() { return &m_axes; }
  VEE_Matrix3 *rotation() { return &m_rotation; }
  // void setCenter(float x, float y, float z);
  // void move(const VEE_Vector3 &l) { m_center += l; }
  void setAxes(float a1, float a2, float a3);
  void setRotation(VEE_Matrix3 &rot) { m_rotation = rot; }
  void setRandom(VEE_Random *r) { m_random = r; }


protected:

  // VEE_Vector3  m_center;
  VEE_Vector3  m_axes;
  VEE_Matrix3  m_rotation; 

  VEE_ClonablePointer<VEE_Random> m_random; 
};


#endif
