
/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_volume_sphere.h>

#include <vee_bounding_box.h>

VEE_VolumeSphere::VEE_VolumeSphere()
  : m_innerRadius(1.0),
    m_outerRadius(2.0)
{}

VEE_VolumeSphere::~VEE_VolumeSphere()
{}

float VEE_VolumeSphere::inside(const VEE_Vector3 &loc)
{
  float l = loc.length();

  if(l <= m_innerRadius) return 1.0f;
  else if(l >= m_outerRadius) return 0.0f;
  
  float d = m_outerRadius - m_innerRadius;
  
  return (l - m_innerRadius) / d;
}

void VEE_VolumeSphere::getBoundingBox(VEE_BoundingBox *box)
{
  box->high().make(m_outerRadius, m_outerRadius, m_outerRadius);
  box->low() = - box->high();
}

VEE_Volume * VEE_VolumeSphere::clone() const
{
  return new VEE_VolumeSphere(*this);
}
