/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_VORTEX_H
#define VEE_VORTEX_H

#include <vee_object.h>
#include <vee_scene.h>

/** Makes other particles rotate around given axes.
*/

class VEE_Vortex : public VEE_ForceObject
{
public:
  VEE_Vortex();

  virtual ~VEE_Vortex();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Variable getVariable(const char *) const;

  virtual VEE_Object * clone() const;

  float epsilon() const { return m_epsilon; }
  float magnitude() const { return m_magnitude; }

  void setEpsilon(float e) { m_epsilon = e; }
  void setMagnitude(float m) { m_magnitude = m; }
  void setRange(float r) { m_range = r; }
  void setHeight(float h) { m_height = h; m_useHeight = true; }
  float getRange() const { return m_range; }
  float height() const { return m_height; }
  double vortexed() const { return m_vortexed; }

  void setAxes(float a1, float a2, float a3);
  void setAxes(const VEE_Vector3 &axes)
  { m_axes = axes; m_axes.normalize(); }
  VEE_Vector3 & getAxes() { return m_axes; }

  virtual void update(float dt);

  void getForce(VEE_Vector3 *force,
		const VEE_Vector3 &velocity,
                const VEE_Vector3 &location);
  void boundingBoxOfForce(VEE_BoundingBox *box);

  static VEE_Object * factory();

protected:

  VEE_Vector3 m_axes;

  float m_magnitude;
  float m_epsilon;
  float m_range;
  float m_height;
  float m_fadeRange;
  float m_maxForce;
  float m_strength;
  float m_radialForce;
  float m_minRadius;

  double m_vortexed;
  /** If this is false, then any radius change will also set the
      height. Any height change via "setValue" will as a side effect
      switch this value to true. */
  bool  m_useHeight;
};

#endif
