/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_MAGIC_RAIN_H
#define VEE_MAGIC_RAIN_H

#include <vee_color_table.h>
#include <vee_particle.h>
#include <vee_random.h>

class VEE_VolumeDisk;

class VEE_MagicRainDrop : public VEE_Particle
{
public:
  VEE_MagicRainDrop(VEE_Vector3 location, 
		    VEE_Vector3 velocity,
		    float ageLimit, 
		    VEE_Looks *l,
		    float childSpeed,
		    float childAgeLimit,
		    float creationRate);

  virtual ~VEE_MagicRainDrop() {}

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Object * clone() const;

  virtual void update(float dt);
  
  void setChildrensInitialSpeed(float speedMin, float speedMax) 
  { m_childSpeedMin = speedMin; m_childSpeedMax = speedMax; }

  void setCreationRate(float rate) { m_creationRate = rate; }
  
  uint childCount() const { return m_childCount; }

  void setVelocityUpdate(bool p) { m_updateVelocity = p; }

protected:

  void createNewParticle();
  
  float m_childSpeedMin;
  float m_childSpeedMax;
  float m_childAgeLimit;
  float m_creationRate;

  float m_sinceLastCreated;

  VEE_RandomUniform m_rand;

  uint m_childCount;

  VEE_Matrix3 m_rotation;

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Magical rain effect/particle for VEE. 
*/

class VEE_MagicRain : public VEE_Particle
{
public:
  VEE_MagicRain(VEE_Vector3 location, 
		VEE_Vector3 velocity,
		float ageLimit, 
		float radius,
		float childSpeed,
		float childRate);

  virtual ~VEE_MagicRain();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Object * clone() const;

  virtual void update(float dt);

  virtual void render(VEE_Renderer *);
  
  void setCreationRate(float r)
  { m_childRate = r; }

  
protected:

  VEE_RgbaTable m_colorTable;

  VEE_VolumeDisk *m_volume;

  VEE_RandomUniform m_rand;

  float m_childSpeed;
  float m_childRate;
  float m_sinceLastCreated;
};

#endif
