/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */


#ifndef VEE_POLYGON_FIRE_H
#define VEE_POLYGON_FIRE_H

#include <vee_color_table.h>
#include <vee_particle.h>
#include <vee_random.h>

/** Fire made of polygons. This class outputs polygons (flames),
    vortex-groups (animations) and gusts (gravitates). */
class VEE_PolygonFire : public VEE_Particle
{
public:
  VEE_PolygonFire(float flameHz = 20.0,
		  float flameAge = 4.0,
		  float vortexHz = 3.0,
		  float gustHz = 1.0);
  virtual ~VEE_PolygonFire();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Object * clone() const;

  virtual void update(float dt);
  
protected:

  virtual void emitFlame();
  virtual void emitGust();
  virtual void emitVortex();

  VEE_RgbaTable m_colorTable;

  float m_flameInterval;
  float m_sinceLastFlame;
  float m_flameAge;

  float m_vortexInterval;
  float m_sinceLastVortex;

  float m_gustInterval;
  float m_sinceLastGust;

  VEE_RandomUniform m_rand;

};

#endif
