/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_COLLECTOR_RENDERER_H
#define GEE_COLLECTOR_RENDERER_H

#include <gee_renderer.h>
#include <gee_transparency_list.h>

#include <list>

class GEE_Geometry;

class GEE_CollectorRenderer : public GEE_Renderer
{
public:

  GEE_CollectorRenderer();
  virtual ~GEE_CollectorRenderer();

  virtual void renderGroupStructure(GEE_Group *);
  virtual void renderLines3D(GEE_Lines3D *);
  virtual void renderPath3D(GEE_Path3D *);
  virtual void renderParticleObject(GEE_ParticleObject *);
  virtual void renderPolygonMesh(GEE_PolygonMesh *);
  virtual void renderProfile(GEE_Profile *);
  virtual void renderSphere(GEE_Sphere *);
  virtual void renderTriangleMesh(GEE_TriangleMesh *);
  virtual void renderVeeObject(VEE_Object *, 
			       const VEE_Matrix4 * m = 0, 
			       float scale = 0.0f);

  virtual void pushTransform(const VEE_Matrix4d &);
  virtual void pushTransform(const VEE_Matrix4 &);
  virtual void popTransform();
  virtual void enableTransforms(bool enable);
  virtual bool enableTransforms();
  
  void clear();

  GEE_TransparencyList & transparencies() { return m_transparencies; }
  GEE_TransparencyList & solids()         { return m_solids; }

  void setCameraLocation(const VEE_Vector3 &l)
  {
    m_transparencies.setCameraLocation(l);
    m_solids.setCameraLocation(l);
  }

protected:
  GEE_TransparencyList m_transparencies;
  GEE_TransparencyList m_solids;

  enum {  STACK_MAXIMUM = 64 };

  VEE_Matrix4d         m_stack[STACK_MAXIMUM];
  VEE_Matrix4          m_current;
  int                  m_stackIndex;
};


#endif
