/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_GALLERY_H
#define GEE_GALLERY_H

#include <gee_world.h>

#include <vee_reference_object.h>

#include <map>

/** A gallery containing all kinds of models. A gallery contains
    multiple worlds, with each world having a set of geometry in
    it. Each world is basically a model.

    @author Tommi Ilmonen.
*/

class GEE_Gallery
{
public:
  typedef std::map<GEE_id_t, VEE_RefPtr<GEE_World> > container;
  typedef container::iterator iterator;

  GEE_Gallery();
  ~GEE_Gallery();

  GEE_id_t takeSnapshot(GEE_World *, 
			GEE_id_t layer = -1,
			bool onlySelected = false);

  GEE_World * item(GEE_id_t id);
  VEE_RefPtr<GEE_World> * item2(GEE_id_t id);
  
  void clear();

  bool read(GEE_Io *);
  bool write(GEE_Io *);
  bool writeItem(const char *file, GEE_id_t id);

  void collectIds(GEE_IdContainer::id_remap &);
  void remapIds(GEE_IdContainer::id_remap &);

  void merge(GEE_Gallery &);

  iterator begin() { return m_worlds.begin(); }
  iterator end()   { return m_worlds.end(); }
  iterator getIndex(uint);
  void erase(iterator it);

  void setGarbageBin(VEE_GarbageBin * bin);

  uint size() { return m_worlds.size(); }

  static VEE_RefPtr<GEE_World> & getWorld(iterator & it) 
  { return (*it).second; }
  
  GEE_id_t itemId(int index);

protected:
  container m_worlds;
  VEE_GarbageBin * m_deleted;
};

#endif
