/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_GROUP_H
#define GEE_GROUP_H

#include <gee_geometry.h>

#include <vee_reference_object.h>

#include <set>

class GEE_Group : public GEE_Geometry
{
public:
  typedef id_container::iterator iterator;

  GEE_Group();
  virtual ~GEE_Group();

  virtual id_container * getExtraIds();

  static GEE_Geometry * create();
  virtual GEE_Geometry * clone() const;
  virtual GEE_Geometry * copy(GEE_World *target, id_remap * remap) const;
  virtual void erase();
  virtual bool eraseData(GEE_Containment *);
  virtual bool intersects(GEE_Containment *);
  virtual void selectVertices(GEE_Containment *,  
			      uint mask, 
			      uint values);
  virtual void selectFaces(GEE_Containment *,  
			   uint mask, 
			   uint values);
  virtual bool colorData(GEE_ReColor *);
  virtual bool moveData(GEE_Mover *);
  virtual void render(GEE_Renderer *);
  virtual bool getBoundingBox(VEE_BoundingBox *);
  virtual bool move(const VEE_Vector3 & offset);
  virtual void setMaterial(const GEE_Material &m);

  virtual bool read(GEE_Io * );
  virtual bool write(GEE_Io *);
  virtual bool finalizeRead();

  virtual void setSelectionState(uint s);

  void clear();
  
  void addChild(GEE_id_t id);
  void breakGroup();

  uint childCount() const { return m_children.size(); }

  iterator begin() { return m_children.begin(); }
  iterator end()   { return m_children.end(); }

  static GEE_id_t getGeometryId(iterator & it) { return  *it; }
  // static std::string getGeometryName(iterator & it) { return  (*it).first; }
  VEE_RefPtr<GEE_Geometry> * getGeometry(iterator & it);

  bool loadModel(const char *filename, bool verbose = true);

  bool loadLightwaveObj(const char *filename, bool verbose = true);
  bool loadWavefrontObj(const char *filename, bool verbose = true);

  void convertPolygonsToTriangles();

  uint size() const { return m_children.size(); }

protected:

  id_container m_children;

};

#endif
