/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_group_around_path.h>

#include <gee_io.h>
#include <gee_group.h>
#include <gee_macros.h>
#include <gee_world.h>

#include <vee_error.h>

GEE_GroupAroundPath::GEE_GroupAroundPath(GEE_id_t pathId)
  : GEE_EraseAroundPath(pathId),
    m_groupPtr(0),
    m_groupId(-1)
{
  addIdMember(&m_groupId);
}

GEE_GroupAroundPath::~GEE_GroupAroundPath()
{}

bool GEE_GroupAroundPath::stop(GEE_World *w, const GEE_TimeStamp &now)
{
  GEE_EraseAroundPath::stop(w, now);
  m_groupPtr = 0;
  m_group.breakLink();
  return true;
}

static const ushort GEE_GroupId   = GEE_CONST2('G', 'I');
static const ushort GEE_EndId     = GEE_CONST2('E', 'N');

bool GEE_GroupAroundPath::read(GEE_Io *io)
{
  if(!GEE_EraseAroundPath::read(io))
    return false;

  ushort id = io->readInt16();

  while(id != GEE_EndId) {
    if(id == GEE_GroupId)
      m_groupId = io->readInt64();
    else {
      VEE_error("GEE_GroupAroundPath::read # Bad id at %d (%c%c)", 
		(int) io->position(), (uchar) (id >> 8), (uchar) id);
      return false;
    }
    id = io->readInt16();
  }

  return true;
}

bool GEE_GroupAroundPath::write(GEE_Io *io)
{
  GEE_EraseAroundPath::write(io);

  io->writeInt16(GEE_GroupId);
  io->writeInt64(m_groupId);

  io->writeInt16(GEE_EndId);

  return true;
}

const char * GEE_GroupAroundPath::type()
{
  return "GEE_GroupAroundPath";
}

GEE_Operator * GEE_GroupAroundPath::create()
{
  return new GEE_GroupAroundPath;
}

void GEE_GroupAroundPath::doWork(GEE_World *w)
{
  GEE_Geometry * g;
  // VEE_trace("GEE_GroupAroundPath::doWork");

  for(GEE_LayerGeometryIterator it(w, m_layer); (g = *it); it.nextVisible()) {
    if(g->intersects(this)) {
      
      /* VEE_trace("GEE_GroupAroundPath::doWork # Intersected geometry %d",
	 (int) g->id()); */

      if(!m_groupPtr) {
	m_groupPtr = new GEE_Group;
	m_groupPtr->setId(m_groupId);
	m_groupPtr->setLayer(m_layer);
	m_group = m_groupPtr;
	m_groupId = w->addGeometry2(m_group);
	/* VEE_trace("GEE_GroupAroundPath::doWork # Created group %d",
	   (int) m_groupPtr->id()); */
      }
      
      // VEE_trace("GEE_GroupAroundPath::doWork # Getting top id");

      GEE_id_t topId = g->masterGroup();
      
      if(topId != m_groupPtr->id()) {

	m_groupPtr->addChild(topId);
	/* VEE_trace("GEE_GroupAroundPath::doWork # Added geo %d to group %d",
	   (int) topId, (int) m_groupPtr->id()); */
      }
    }
  }
  // VEE_trace("GEE_GroupAroundPath::doWork # EXIT");
}
