/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_group_geos.h>

#include <gee_group.h>
#include <gee_io.h>
#include <gee_macros.h>
#include <gee_world.h>

#include <vee_error.h>

GEE_GroupGeos::GEE_GroupGeos(GEE_id_t groupId)
  : m_groupId(groupId)
{
  addIdMember(&m_groupId);
}

GEE_GroupGeos::~GEE_GroupGeos()
{}

GEE_IdContainer::id_container * GEE_GroupGeos::getExtraIds()
{
  return & m_ids;
}

bool GEE_GroupGeos::act(GEE_World * w, const GEE_TimeStamp &)
{
  if(m_groupId < 0)
    m_groupId = allocateId();

  GEE_Group * group = new GEE_Group();

  group->setId(m_groupId);

  w->addGeometry(group);

  for(id_container::iterator it = m_ids.begin(); it != m_ids.end(); it++) {

    GEE_id_t id = *it;
    GEE_Geometry * g = w->geometry(id);
    
    if(g && (g != group)) {
      group->addChild(id);
      VEE_trace("GEE_GroupGeos::act # Grouped %d (%p) to %d (%p)", 
		(int) id, g, (int) m_groupId, group);
    }
    else
      VEE_error("GEE_GroupGeos::act # Grouping %d (%p) to %d (%p) failed", 
		(int) id, g, (int) m_groupId, group);
  }

  return true;
}

static const ushort GEE_IdId       = GEE_CONST2('I', 'D');
static const ushort GEE_EndId      = GEE_CONST2('E', 'N');

bool GEE_GroupGeos::read(GEE_Io *io)
{
  std::string str;
  io->readNullTerminatedStdString(str);
  if(str != type())
    return false;
  
  readBasics(io);

  int v = io->readInt32();

  if(v != 1)
    return false;

  m_ids.clear();

  m_groupId = io->readInt64();

  ushort id = io->readInt16();

  while(id != GEE_EndId) {
    if(id == GEE_IdId)
      addId(io->readInt64());
    else {
      VEE_error("GEE_GroupGeos::read # Bad id at %d (%c%c)", 
		(int) io->position(), (uchar) (id >> 8), (uchar) id);
      return false;
    }
    id = io->readInt16();
  }
  return true;
}

bool GEE_GroupGeos::write(GEE_Io *io)
{
  io->writeNullTerminatedString(type());

  writeBasics(io);

  io->writeInt32(1); // version
 
  io->writeInt64(m_groupId);

  for(id_container::iterator it = m_ids.begin(); it != m_ids.end(); it++) {
    io->writeInt16(GEE_IdId);
    io->writeInt64(*it);
  }
  io->writeInt16(GEE_EndId);

  return true;
}

const char * GEE_GroupGeos::type()
{
  return "GEE_GroupGeos";
}

GEE_Operator * GEE_GroupGeos::create()
{
  return new GEE_GroupGeos();
}
