/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_HISTORY_ANIMATION_H
#define GEE_HISTORY_ANIMATION_H

#include <gee_id.h>
#include <gee_time_stamp.h>

#include <vee_time.h>

class GEE_Engine;
class GEE_OperatorGroup;

/** Makes an animation of the creation history. 

    @author Tommi Ilmonen.
*/

class GEE_HistoryAnimation
{
public:
  GEE_HistoryAnimation();
  ~GEE_HistoryAnimation();

  void start(GEE_Engine *);

  void update(float delta = 0.0f);
  
  void stop();

  bool isDone();

protected:

  void startNewRound(GEE_id_t groupToInclude);
  void endRound();

  typedef std::set<GEE_id_t> container;
  typedef container::iterator iterator;

  container m_groupIds;
  iterator  m_iterator;

  GEE_Engine        * m_engine;
  GEE_OperatorGroup * m_currentGroup;
  /// Scale of the current operator group:
  double              m_currentScale;
  /// Current animation time
  GEE_TimeStamp       m_currentTime;

  VEE_Time  m_prevTime;
};

#endif
