/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_id_select.h>

#include <gee_io.h>
#include <gee_macros.h>
#include <gee_world.h>

#include <vee_error.h>

#include <typeinfo>

GEE_IdSelect::GEE_IdSelect()
{}

GEE_IdSelect::~GEE_IdSelect()
{}

GEE_IdSelect::id_container * GEE_IdSelect::getExtraIds()
{
  return & m_ids;
}

bool GEE_IdSelect::act(GEE_World *w, const GEE_TimeStamp &)
{
  for(id_container::iterator it = m_ids.begin(); it != m_ids.end(); it++) {
    GEE_Geometry * d = w->geometry((*it));
    if(d)
      d->setSelectionState(GEE_SELECT_BIT);

    VEE_trace("GEE_IdSelect::act # %d %s", (int) (*it), typeid(*d).name());
  }

  return true;
}

static const ushort GEE_IdId       = GEE_CONST2('I', 'D');
static const ushort GEE_EndId      = GEE_CONST2('E', 'N');

bool GEE_IdSelect::read(GEE_Io *io)
{
  std::string str;
  io->readNullTerminatedStdString(str);
  if(str != type())
    return false;
  
  readBasics(io);

  int v = io->readInt32();

  if(v != 1)
    return false;

  ushort id = io->readInt16();

  while(id != GEE_EndId) {
    if(id == GEE_IdId)
      addId(io->readInt64());
    else {
      VEE_error("GEE_IdSelect::read # Bad id at %d (%c%c)", 
		(int) io->position(), (uchar) (id >> 8), (uchar) id);
      return false;
    }
    id = io->readInt16();
  }
  
  return true;
}

bool GEE_IdSelect::write(GEE_Io *io)
{
  io->writeNullTerminatedString(type());

  writeBasics(io);

  io->writeInt32(1); // version
 
  for(id_container::iterator it = m_ids.begin(); it != m_ids.end(); it++) {
    io->writeInt16(GEE_IdId);
    io->writeInt64(*it);
  }
  io->writeInt16(GEE_EndId);
  return true;
}

const char * GEE_IdSelect::type()
{
  return "GEE_IdSelect";
}

GEE_Operator * GEE_IdSelect::create()
{
  return new GEE_IdSelect;
}
