/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_insert_particle.h>

#include <gee_io.h>
#include <gee_macros.h>
#include <gee_world.h>

GEE_InsertParticle::GEE_InsertParticle()
  : m_objId(-1)
{
  addIdMember( & m_objId);
  m_writer.addMemberId(GEE_CONST2('O', 'I'), & m_objId);
}

GEE_InsertParticle::~GEE_InsertParticle()
{}
  
bool GEE_InsertParticle::act(GEE_World * w, const GEE_TimeStamp &)
{
  GEE_ParticleObject * po = new GEE_ParticleObject();
  
  po->setLayer(m_layer);
  po->setId(m_objId);
  po->addParticle(m_particle);

  w->addGeometry(po);

  m_objId = po->id();

  return true;
}

bool GEE_InsertParticle::read(GEE_Io *io)
{
  std::string str;
  io->readNullTerminatedStdString(str);
  if(str != type())
    return false;

  readBasics(io);

  int v = io->readInt32();

  if(v != 1)
    return false;

  m_writer.readParameters(io);

  return m_particle.read(io);
}

bool GEE_InsertParticle::write(GEE_Io *io)
{
  io->writeNullTerminatedString(type());

  writeBasics(io);
  
  io->writeInt32(1); // version

  m_writer.writeParameters(io);

  m_particle.write(io);

  return true;
}
 
const char * GEE_InsertParticle::type() 
{ 
  return "GEE_InsertParticle"; 
}

GEE_Operator * GEE_InsertParticle::create()
{
  return new GEE_InsertParticle();
}
