/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_interval.h>

#include <gee_debug.h>
#include <gee_io.h>
#include <gee_macros.h>

#include <vee_error.h>

static const int GEE_IntervalId = GEE_CONST('I', 'N', 'T', 'V');

bool GEE_Interval::read(GEE_Io *io)
{
  if(io->readInt32() != GEE_IntervalId)
    return false;

  int v = io->readInt32(); // Version

  m_begin = io->readInt64();
  m_end   = io->readInt64();

  if(v != 1) {
    VEE_error("GEE_Interval::read # Bad version (%d)", v);
    return false;
  }
  
  if(GEE_Debug::debugAll())
    VEE_trace("GEE_Interval::read # %lf %lf",
	      m_begin.secondsD(), m_end.secondsD());

  return true;
}

bool GEE_Interval::write(GEE_Io *io)
{
  io->writeInt32(GEE_IntervalId);
  io->writeInt32(1); // Version
  io->writeInt64(m_begin.stamp());
  io->writeInt64(m_end.stamp());

  if(GEE_Debug::debugAll())
    VEE_trace("GEE_Interval::write # %lf %lf", 
	      m_begin.secondsD(), m_end.secondsD());

  return true;
}
