/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_material.h>

#include <gee_io.h>
#include <gee_macros.h>


GEE_Material::GEE_Material(float shine, 
			   const VEE_Vector4 & color, 
			   int flags)
  : m_shine(shine), 
    m_ambient(color), 
    m_diffuse(color), 
    m_specular(color),
    m_burn(0.0),
    m_flags(flags)
{
  if(color[3] != 1.0f)
    m_flags |= GEE_MAT_BLEND;
}

GEE_Material::~GEE_Material()
{}

static const int GEE_MaterialId = GEE_CONST('M', 'A', 'T', 'L');

bool GEE_Material::read(GEE_Io *io)
{
  if(io->readInt32() != GEE_MaterialId)
    return false;

  int v = io->readInt32();

  m_shine    = io->readFloat32();
  m_ambient  = io->readVector4();
  m_diffuse  = io->readVector4();
  m_specular = io->readVector4();

  m_flags    = io->readInt32();
  
  io->readNullTerminatedStdString(m_texture);

  return v == 1;
}

bool GEE_Material::write(GEE_Io *io)
{
  io->writeInt32(GEE_MaterialId);
  io->writeInt32(1); // Version  

  io->writeFloat32(m_shine);
  io->writeVector4(m_ambient);
  io->writeVector4(m_diffuse);
  io->writeVector4(m_specular);

  io->writeInt32(m_flags);
  
  io->writeNullTerminatedString(m_texture.c_str());

  return true;
}

bool GEE_Material::operator == (const GEE_Material & that)
{
  return m_shine == that.m_shine &&
    m_ambient  == that.m_ambient &&
    m_diffuse  == that.m_diffuse &&
    m_specular == that.m_specular &&
    m_flags    == that.m_flags &&
    m_texture  == that.m_texture;    
}
