/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_MOVE_AROUND_PATH_H
#define GEE_MOVE_AROUND_PATH_H

#include <gee_mover.h>
#include <gee_path_to_x.h>

#include <vector>

class GEE_MoveAroundPath : public GEE_PathToX, public GEE_Mover
{
public:

  enum Algorithm {
    ALGO_OLD_BUGGY,
    ALGO_NEW1
  };

  GEE_MoveAroundPath(GEE_id_t pathId = -1, 
		     GEE_id_t geoId = -1, 
		     bool rotate = false,
		     bool useNapra = false,
		     float radiusNorm = 1.0f,
		     float exaggerateRadius = 0.0f);
  virtual ~GEE_MoveAroundPath();
  
  virtual id_container * getExtraIds();

  virtual bool act(GEE_World *, const GEE_TimeStamp &);

  virtual bool read(GEE_Io *);
  virtual bool write(GEE_Io *);
  virtual const char * type();
  static GEE_Operator * create();

  virtual void getBoundingBox(VEE_BoundingBox *);
  /// From GEE_Mover
  virtual bool move(VEE_Vector3 *);
  virtual void moved(GEE_Geometry *);

  void setParams(float radius, float pressure)
  { m_radius = radius; m_pressure = pressure; }

  void setRadius(float radius) { m_radius = radius; }

  void addGeometryId(GEE_id_t geoId) { m_geoIds.insert(geoId); }

protected:

  virtual bool handleNewVertex(GEE_Path3D::iterator &);

  virtual void doMove(GEE_World *);

  class Move
  {
  public:
    Move() : m_radius(1.0f) {}

    VEE_Vector3 m_origin;
    VEE_Vector3 m_move;
    VEE_Matrix3d m_rotate;
    float        m_radius;
  };

  VEE_Vector3          m_curLoc;
  VEE_Matrix3d         m_curRot;

  std::vector<Move>    m_moves;
  VEE_BoundingBox      m_bbox;

  float                m_radius;
  float                m_exaggerateRadius;
  float                m_radiusNorm;
  float                m_pressure;
  int                  m_algorithm;

  /** The id of the geometry to be moved. If id is -1, then we move
      vertices. */
  id_container m_geoIds;
  bool     m_rotate;
  bool     m_useNapra;
};


#endif
