/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_PATH_MULTIPLY_H
#define GEE_PATH_MULTIPLY_H

#include <gee_path_to_x.h>
#include <gee_profile.h>

#include <vee_reference_object.h>

class GEE_PathMultiply : public GEE_PathToX
{
public:
  GEE_PathMultiply(GEE_id_t pathId = -1);
  virtual ~GEE_PathMultiply();

  virtual id_container * getExtraIds();

  virtual bool start(GEE_World *, const GEE_TimeStamp &);
  virtual bool stop(GEE_World *, const GEE_TimeStamp &);

  virtual bool read(GEE_Io *);
  virtual bool write(GEE_Io *);
  virtual const char * type();

  static GEE_Operator * create();

  void setProfile(const GEE_Profile &profile);  

  void setScale(float scale) { m_scale = scale; }
  void setThreshold(float t) { m_threshold = t; }

  void setAlignment(GEE_AlignMode m) { m_align = m; }

  uint pathCount() const { return m_profile.size(); }
  GEE_id_t pathId(int index);

  // void useNapra(bool withNapra) { m_withNapra = withNapra; }

  void setExtrude(const VEE_Vector3 & ext) { m_extrude = ext; }

protected:
  virtual bool handleNewVertex(GEE_Path3D::iterator &);

  GEE_ParamWriter m_writer;

  VEE_Vector3 m_prevLoc;
  VEE_Vector3 m_dir;
  VEE_Vector3 m_extrude;

  float       m_scale;
  float       m_threshold;
  int         m_align;
  bool        m_newSegment;
  GEE_Profile m_profile;

  id_container m_ids;

  class Item
  {
  public:
    Item() : m_path (0) {}

    VEE_RefPtr<GEE_Geometry>    m_geo;
    GEE_Path3D                * m_path;
    VEE_Vector2                 m_vertex;
  };

  std::vector<Item> m_paths;
};


#endif

