/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <gee_sphere_group.h>

#include <WmlIntrTri3Sph3.h>

GEE_SphereGroup::GEE_SphereGroup()
  : m_radius(1.0)
{
  m_bbox.clear();
}

GEE_SphereGroup::~GEE_SphereGroup()
{}

void GEE_SphereGroup::getBoundingBox(VEE_BoundingBox *bb)
{
  *bb = m_bbox;
}

bool GEE_SphereGroup::contains(const VEE_Vector3 &loc)
{
  VEE_Vector3 l = loc;

  for(std::vector<Sphere>::iterator it = m_locations.begin();
      it != m_locations.end(); it++)
    if((l - (*it).m_location).lengthSqr() <= (*it).m_radiusSquared)
      return true;
  
  return false;
}

bool GEE_SphereGroup::intersects(const GEE_SimpleTriangle &tri)
{
  Wml::Triangle3<float> wtri;

  VEE_Vector3 e0 = tri.m_vertices[2] - tri.m_vertices[0];
  VEE_Vector3 e1 = tri.m_vertices[1] - tri.m_vertices[0];

  wtri.Origin() = * (Wml::Vector3<float> *) & tri.m_vertices[0];
  wtri.Edge0()  = * (Wml::Vector3<float> *) & e0;
  wtri.Edge1()  = * (Wml::Vector3<float> *) & e1;

  Wml::Sphere3<float> sphere;

  for(std::vector<Sphere>::iterator it = m_locations.begin();
      it != m_locations.end(); it++) {
    sphere.Center() = * (Wml::Vector3<float> *) & (*it).m_location;
    sphere.Radius() = (*it).m_radius;
    
    if(Wml::TestIntersection(wtri, sphere))
      return true;
  }
  return false;
}

void GEE_SphereGroup::addSphere(const VEE_Vector3 &loc, float radius)
{
  if(radius < 0.0f)
    radius = m_radius;

  if(!m_locations.size())
    m_bbox.set(loc, radius);
  else
    m_bbox.expand(loc, radius);

  Sphere s;
  s.m_location = loc;
  s.m_radius   = radius;
  s.m_radiusSquared = radius * radius;

  m_locations.push_back(s);
}
