/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_TCB_SPLINE3_H
#define VEE_TCB_SPLINE3_H

#include <WmlTCBSpline3.h>

#include <vee_vector3.h>

/** TCB spline calculator. This class depends on the Mgc::TCBSpline3
    class to do the dirty work. This class helps by enabling tools to
    modify and recalculate the spline after construction. */

class VEE_TCBSpline3 : public Wml::TCBSpline3<float>
{
public:

  VEE_TCBSpline3(int maxknots,
		 float tension = 0.5, 
		 float continuity = 0.5, 
		 float bias = 0.5);
  virtual ~VEE_TCBSpline3();

  void setKnots(int n);
  int  allocatedKnots() const { return m_allocatedKnots; }

  void setKnotParams(int n, float tension, float continuity, float bias);
  void setKnotTime(int n, float t);
  void setPoint(int n, const VEE_Vector3 &);

  void recalculate();

protected:
  int m_allocatedKnots;
};

#endif
