
# Usual initializations:

from vee_init import *
from vee import *
import math

qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

vq = VEE_Qt(scene)
# Make force invisible:
q2 = VEE_TestQt(vq, scene, "", 640, 480, VEE_GL_SL, vq)
q2.setMtlFile("vee_materials.mtl");
vq.setGlWidget(q2);

vq.getGlWidget().setAutoRewind(0);
world = vq.getGlWidget().geeEngine().world();

agelimit = 5.0
gravity = 5.0

# object = "cat.lwo";
object = "cow.obj";
# object = "cupbase.lwo";
# object = "/usr/share/Performer/data/teapot.dxf";
# objectscale = 1.0;

objectField = object + ".vft"

group = GEE_Group();
group.thisown = 0
world.addGeometry(group)
# g.loadModel("/usr/share/Performer/data/dumptruck.obj");
group.loadModel(object);
group.centralize();
# g.loadModel("hakkyra.obj");

rebuildField = 1

if rebuildField:
    vgrid = VEE_VectorGrid();
    vgrid.createFromModel(group, 50, 2, 3, -1);
    vgt = VEE_VectorGridTree()
    vgt.build(vgrid, 4, 0.3)
    vgt.write(objectField);

#############################################################################

vgt = VEE_VectorGridTree()

res = vgt.read(objectField)

# res.sdfasdfdasfaefa() # EXIT

gridforce = VEE_GridForce2()

gridforce.setGrid(vgt)
# gridforce.setValueFloat("magnitude", 20.0);
# gridforce.setValueFloat("scale", objectscale);
# gridforce.setForceType(VEE_FORCE_HIDDEN);
gridforce.setValueInt("forcetype", 2);

scene.addNewObject(gridforce)


#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_Particle()
# p = VEE_TailParticle()
p.thisown = 0

#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;

v1 = VEE_Vector3()
v2 = VEE_Vector3()

range = 0.2

v1.make(-range, -0.5 * range , -range)
v2.make( range,  0.5 * range,  range)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)


#############################################################################

# vortex = VEE_ObjectFactory_create("vortex")
# scene.addNewObject(vortex)
# vortex.setAgeLimit(2000.0)

#############################################################################

# Create a vortex generator:

vortex = VEE_ObjectFactory_create("vortex")
vortex.setAgeLimit(1.5);
vortex.setValueFloat("range", 1.0);
vortex.setValueFloat("magnitude", 0.5);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex)
screator.thisown = 0;
screator.setValueInt("visible", 0);

# Random direction:
vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())
screator.setRate(1.0);

scene.addNewObject(screator)


#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)
g.setValueInt("visible", 0);

#grav = VEE_ObjectFactory_create("gravitate")
#grav.setValueFloat("range", 2.0);
#grav.setValueFloat("magnitude", -200); # Push away from object
#grav.setValueFloat("maxforce", 10);
#grav.setLocation3(1.0, 4.0, 0.0);
#scene.addNewObject(grav)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings for a couple a few parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Gravity", "gravity", g, -5, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid Bounce distance",
                            "bouncedistance", gridforce,
                            0.2, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid flow distance",
                            "flowdistance", gridforce,
                            1.0, 0.0, 4.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid drag distance",
                            "dragdistance", gridforce,
                            0.2, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid magnitude", "magnitude", gridforce,
                            400.0, .5, 1000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Grid bounciness", "bounce", gridforce,
                            0.0, 0.0, 1.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid drag", "drag", gridforce,
                            0.0, 0.0, 7.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid drag power", "dragpower", gridforce,
                            2.0, 0.0, 3.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid tangent force", "tangentforce", gridforce,
                            3.0, -100.0, 100.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid tangent grab", "tangentgrab", gridforce,
                            30.0, -100.0, 100.0,
                            VEE_MAP_LINEAR);



paramView.addFloatParameter("Flame agelimit", "agelimit", p, 4, .5, 10.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame tail thresh", "threshold", p, 0.2, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 300.0, 0.0, 500.0,
                            VEE_MAP_LINEAR);

paramView.addVector3Parameter("Flame location", "location", creator,
                              1.0, -7.0, 7.0,
                              3.0, -3.0, 7.0,
                              0.0, -7.0, 7.0);

paramView.addFloatParameter("Vortex rate", "rate", screator, 0.0, 0.0, 3.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Vortex range", "range", vortex, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, 3.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex maxforce", "maxforce", vortex, 10.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);


#############################################################################

# Make it visible:

vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()
# vq.setRenderOrder("1");
vq.setValueFloat("GL/size", 0.04);
vq.setValueInt("GL/subtexture", 5);

vq.setFrameFile("frames/cow-%.5d.jpg")

kp = vq.addKeyPoint2(-0.000, 1.090, 3.762, 3.142, -0.300);
vq.selectKeyPoint2(kp);

paramView.bangValues()

paramView.show()

qa.execute()

