# Simple fire effect

# Usual initializations:
from vee_init import *
from vee import *
import math
qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()
GEE_World_loadTextureDefinitions("vee-gl.config");

agelimit = 2.0
gravity = 5.0

ts = TimeStamp()

# GEE_Debug_setDebug("GEE_OperatorManager", 2);

#############################################################################

rendertype = VEE_RenderClass();

#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_Particle2()
p.thisown = 0

p.setRenderClass(rendertype);

# p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(1.0, 0.1, 0.0, 0.5, 0.9)
rgbatable.appendColor(color)
color.set(1.0, 1.0, 0.3, 0.5, 0.8)
rgbatable.appendColor(color)
color.set(0.7, 0.7, 0.2, 0.5, 0.5)
rgbatable.appendColor(color)
color.set(0.4, 0.4, 0.4, 0.7, 0.2)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.4, 0.0, 0.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)

sizecurve = VEE_ValueCurve2();
sizecurve.setId("size");
sizecurve.appendPoint(0.3, 0.0);
sizecurve.appendPoint(0.3, 0.2);
sizecurve.appendPoint(0.4, 0.6);
sizecurve.appendPoint(1.0, 1.0);
sizecurve.setScaling(VEE_TS_AGE_MATCH);

p.appendSelfOperator(sizecurve.clone());

#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;
creator.setVisible(0)

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

v1.make(-2.0, 0.0, -2.0)
v2.make(2.0, 0.5, 2.0)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)


#############################################################################

# Create a vortex generator:

vortex = VEE_ObjectFactory_create("vortex")
vortex.setAgeLimit(1.5);
vortex.setValueFloat("range", 1.0);
vortex.setValueFloat("magnitude", 0.5);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex)
screator.thisown = 0;
screator.setVisible(0)

# Random direction:
vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())
screator.setRate(1.0);

scene.addNewObject(screator)


#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)

wind = VEE_ObjectFactory_create("wind")
scene.addNewObject(wind)

#grav = VEE_ObjectFactory_create("gravitate")
#grav.setValueFloat("range", 2.0);
#grav.setValueFloat("magnitude", -200); # Push away from object
#grav.setValueFloat("maxforce", 10);
#grav.setLocation3(1.0, 4.0, 0.0);
#scene.addNewObject(grav)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings fo a couple a few parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Flame agelimit", "agelimit", p, 6, .5, 6.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame omega", "omega", p, 0, -30, 30,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Gravity", "gravity", g, gravity, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 30.0, 1.0, 4000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addVector3Parameter("Flame location", "location", creator,
                              0.0, -5.0, 5.0,
                              0.0, -5.0, 5.0,
                              0.0, -5.0, 5.0);

paramView.addFloatParameter("Vortex rate", "rate", screator, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex range", "range", vortex, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, 3.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex agelimit", "agelimit", vortex, 3, .5, 6.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.5, -2, 2,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind strength", "strength", wind, 3.0, 0.0, 10,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gustiness", "gustiness", wind, 3.0, 0.0, 20,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gust frequency", "gustfreq",
                            wind, 0.35, 0.1, 20,
                            VEE_MAP_LOGARITHMIC);

#############################################################################

# Make a path model

pa = GEE_PathInsert()
path = pa.path()

rot = VEE_Matrix3()
rot.identity();

path.addVertex(VEE_Vector3(0.0, 2.0, 0.0), rot);
path.addVertex(VEE_Vector3(0.0, 2.0, 2.0), rot);
path.addVertex(VEE_Vector3(0.0, 3.0, 4.0), rot);
path.addVertex(VEE_Vector3(0.0, 3.0, 5.0), rot);
path.addVertex(VEE_Vector3(1.0, 3.0, 6.0), rot);
path.addVertex(VEE_Vector3(2.0, 4.0, 7.0), rot);

n = 30;
for i in range(0,n):
    path.addVertex(VEE_Vector3(3.0 * math.cos(VEE_TWO_PI * i / n),
                               2 + (i * 2.5) / n ,
                               3.0 * math.sin(VEE_TWO_PI * i / n)), rot)

path.addVertex(VEE_Vector3(0.0, 8.0, 0.0), rot);

erasetime = 4.0

group1 = GEE_OperatorGroup()
group1.setScale(0.05);
ts.setSeconds(1.5);
group1.setOffset(ts);
group1.autoId();

group2 = GEE_OperatorGroup()
ts.setSeconds(erasetime);
group2.setOffset(ts);
group2.setScale(0.0);
group2.autoId();

pa2 = GEE_PathInsert()
path2 = pa2.path()
# One of the preceding control points:
path2.addVertex(VEE_Vector3(0.0, 8.0, 0.0), rot); 
rot.rotateZ(0.5);
path2.addVertex(VEE_Vector3(1.0, 3.0, 6.0), rot); 
rot.rotateZ(0.9);
path2.addVertex(VEE_Vector3(1.0, 3.0, 6.0), rot); 
rot.rotateZ(1.6);
path2.addVertex(VEE_Vector3(0.0, 7.0, 7.0), rot); 

pa3 = GEE_PathInsert()
path3 = pa3.path()
# One of the preceding control points:
path3.addVertex(VEE_Vector3(3.0, 4.0, 7.0), rot); 
path3.addVertex(VEE_Vector3(4.0, 7.0, 7.0), rot); 

profile = GEE_Profile()
profile.generateCircle(0.4, 12);
# profile.generateBar(0.4, 0.3);
# profile.getVertex(2).m_continues = 0;
#profile.getVertex(4).m_continues = 0;
#profile.getVertex(6).m_continues = 0;

#############################################################################

# Make it visible:

# vq = VEE_Qt(scene, VEE_GL_CG)
vq = VEE_Qt(scene, VEE_GL_SL)
engine = vq.getGlWidget().geeEngine()
engine.actions().addOperatorGroup(group1);
engine.actions().addOperatorGroup(group2);
world = engine.world()

# profileId = world.addGeometry(profile)

# Configure GEE engine.
pathId = GEE_IdContainer_allocateId();
pa.setPathId(pathId)
engine.addOperator(pa)

pathId2 = GEE_IdContainer_allocateId();
pa2.setPathId(pathId2)
engine.addOperator(pa2)

pathId3 = GEE_IdContainer_allocateId();
pa3.setPathId(pathId3)
engine.addOperator(pa3)

meshId = GEE_IdContainer_allocateId();

light = 0.8
mat = GEE_Material()
mat.setTwoSided(1)
mat.setColor(VEE_Vector4(0.0, light, light, 1.0))

apm = GEE_PathToMesh(pathId, meshId)
apm.setProfile(profile);
apm.setMaterial(mat);
#apm.setThreshold(1.8)
#apm.setAlignment(GEE_ALIGN_ROTATION)
#apm = GEE_PathToLines(pathId, meshId)
#apm.setWidth(3)
#apm = GEE_PathToParticles(pathId, meshId)
#apm.setTextureName("arrow");
#apm.setRadius(1.0);
#apm.setTextureName("noise-1");

apm.setGroup(group1.id());

#eraser = GEE_EraseAroundPath(pathId2)
#eraser.setRadius(0.5);
#eraser = GEE_ColorAroundPath(pathId2)
#eraser = GEE_ScaleAroundPaths(pathId2, pathId3, meshId, 0)
#eraser = GEE_MoveAroundPath(pathId2, meshId)
#eraser = GEE_MoveAroundPath(pathId2)
#eraser = GEE_TextureAroundPath(pathId2)
#eraser.setTextureName("circles");

eraser = GEE_ParticleRotate(pathId2)

eraser.setGroup(group2.id());

#eraser.setRadius(3.0);

profile.thisown = 0;
pa.thisown = 0;
pa2.thisown = 0;
pa3.thisown = 0;
apm.thisown = 0;
eraser.thisown = 0;

ts.setSeconds(path.endTime() + 0.5)
apm.interval().setEnd(ts)

eraser.interval().setEnd(ts)
ts.setSeconds(erasetime);
eraser.interval().setBegin(ts)

ts.setSeconds(5.0);
ids = GEE_IdSelect()
ids.addId(meshId);
ids.thisown = 0;
ids.setGroup(group1.id());
ids.interval().setBegin(ts)
ts.setSeconds(path.endTime() + 0.5)
apm.interval().setEnd(ts)

sphere = GEE_Sphere()
sphere.thisown = 0

vq.getGlWidget().geeEngine().world().addGeometry(sphere)
vq.getGlWidget().geeEngine().actions().addOperator(apm)
vq.getGlWidget().geeEngine().actions().addOperator(eraser)
# vq.getGlWidget().geeEngine().actions().addOperator(ids)
vq.getGlWidget().geeMaterials().defaultMaterial().setUseVertexColors(1)
# vq.getGlWidget().geeEngine().undo();

# Make forces invisible:
vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()

vq.setFrameFile("frames/fire-%.5d.jpg")

paramView.bangValues()
paramView.show()
paramView.thisown = 0

# rendertype.setShader(vq.renderer().shaderNameToId("shaders/mini"));
rendertype.setShader(vq.renderer().shaderNameToId("shaders/depth-fade"));
# rendertype.setShader(vq.renderer().shaderNameToId("shaders/depth-fade-2"));
# rendertype.setShader(vq.renderer().shaderNameToId("shaders/as-opengl"));
# rendertype.setShader(vq.renderer().shaderNameToId("shaders/depth"));

print vq.renderer().subTextureCount()
vq.setValueWindow(paramView)

qa.execute()

vq.getGlWidget().geeEngine().write("wonder.gee");
