# Simple fire effect

# Usual initializations:
from vee_init import *
from vee import *
import math
qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

agelimit = 2.0
gravity = 5.0

# Performance on pyry (rh 7.1)
# Points: 430
# lines: 430
# Quads (medium, stars): 340
# Quads (large): 250

# Complex (about 1200 particles per second) 75fps

# SPEEDS:
#
# RH 7.1: 320-370
#

#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_TailParticle()
# p = VEE_TailParticle2()
p.thisown = 0

# p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(0.7, 0.0, 0.0, 0.3)
rgbatable.appendColor(color)
color.set(0.7, 0.7, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.0, 0.0, 0.0, 0.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)



#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

v1.make(-2.0, 0.0, -2.0)
v2.make(2.0, 0.5, 2.0)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)


#############################################################################

# Another creator:

creator2 = creator.clone()



# particleage = VEE_ValueRandomFloat()
# particleage.set(0.05, 1.0)
# particleage.thisown = 0
# particleageop = VEE_ValueOperator()
# particleageop.set("agelimit", particleage);
# particleageop.thisown = 0

# creator.appendOperator(particleageop)

# op_sqr = VEE_OperatorSquare()
# op_sqr.setId("agelimit");
# creator.appendOperator(op_sqr.clone())
# creator.appendOperator(op_sqr.clone())

# op_mul = VEE_OperatorMultiplyFloat()
# op_mul.setId("agelimit");
# op_mul.setMul(5.0);
# creator.appendOperator(op_mul.clone())

#############################################################################

# Create a vortex generator:

vortex = VEE_ObjectFactory_create("vortex")
vortex.setAgeLimit(1.5);
vortex.setValueFloat("area", 0.0);
vortex.setValueFloat("range", 1.0);
vortex.setValueFloat("magnitude", 0.5);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex)
screator.thisown = 0;

# vortex.setValueVector3f("axes", 0.0, 1.0, 0.0);

# Random direction:
vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())


scene.addNewObject(screator)

#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)

bigVortex = VEE_ObjectFactory_create("vortex")
bigVortex.setAgeLimit(1000000);
bigVortex.setValueFloat("range", 4.0);
bigVortex.setValueFloat("magnitude", 0.2);
bigVortex.setValueVector3f("axes", 0, 1, 0);
bigVortex.setValueFloat("height", 16.0);
bigVortex.setVelocityUpdate(0)

scene.addNewObject(bigVortex)


#############################################################################

# Make GUI mappings fo a couple a few parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Gravity", "gravity", g, gravity, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame 2 agelimit", "type/agelimit",
                            creator2, 3.0, 0.5, 6.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame 2 tail", "type/tailsize",
                            creator2, 50, 4, 200,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Flame 2 rate", "rate",
                            creator2, 30.0, 1.0, 4000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Flame agelimit", "agelimit", p, 1.5, .5, 6.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame tail", "tailsize", p, 50, 4, 200,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Flame tail vel", "tailvel", p, 0.2, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame tail thresh", "threshold", p, 0.2, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 30.0, 1.0, 4000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addVector3Parameter("Flame location", "location", creator,
                              0.0, -5.0, 5.0,
                              0.0, -5.0, 5.0,
                              0.0, -5.0, 5.0);

paramView.addFloatParameter("Vortex rate", "rate", screator, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex age limit", "agelimit", vortex, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex range", "range", vortex, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex height", "height", vortex, 4.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, 3.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Big Vortex magnitude", "magnitude", bigVortex, 1.4, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.5, -2, 2,
                            VEE_MAP_LINEAR);


#############################################################################

# Make it visible:

vq = VEE_Qt(scene)

vq.setValueInt("GL/subtexture", 5);
# Make force invisible:
vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()

paramView.bangValues()

paramView.show()

print vq.renderer().subTextureCount()

qa.execute()

