2# Simple fire effect

# Usual initializations:
from vee_init import *
from vee import *
import math
qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()


# Build a particle with nice colors, lifespan etc:

p = VEE_Particle2()
p.thisown = 0

# p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(1.0, 0.3, 0.0, 0.5, 0.9)
rgbatable.appendColor(color)
color.set(1.0, 0.5, 0.2, 0.5, 0.8)
rgbatable.appendColor(color)
color.set(0.7, 0.5, 0.2, 0.5, 0.5)
rgbatable.appendColor(color)
color.set(0.4, 0.4, 0.4, 0.7, 0.2)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.4, 0.0, 0.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)

sizecurve = VEE_ValueCurve2();
sizecurve.setId("size");
sizecurve.appendPoint(0.3, 0.0);
sizecurve.appendPoint(0.3, 0.2);
sizecurve.appendPoint(0.4, 0.6);
sizecurve.appendPoint(1.0, 1.0);
sizecurve.setScaling(VEE_TS_AGE_MATCH);

p.appendSelfOperator(sizecurve.clone());

#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;
creator.setVisible(0)

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

v1.make(-2.0, -2.0, -2.0)
v2.make(2.0, 2.0, 2.0)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)

wind = VEE_ObjectFactory_create("wind")
scene.addNewObject(wind)

#############################################################################

# Create the grid

vg = VEE_VolumeGrid2()
vg.setValueFloat("scale", 5.0);
vg.setValueInt("division", 15);

v = VEE_Vector3()

v.make(0, 0, 0);
vg.addBall(v, 1.0)
v.make(1.0, 0.0, 0.0)
vg.addBall(v, 1.0)
v.make(0.0, 3.3, 0.0)
vg.addBall(v, 0.7)
v.make(40.0, 0.0, 0.0)
vg.addBall(v, 2.7)
v.make(0.0, 3.3, 3.0)
vg.addBall(v, 0.7)


vg.triangulate()

vvolume = VEE_ValueVolume()
vvolume.setObject(vg.clone())

wind.setValue("volume", vvolume);


#############################################################################

# Make GUI mappings for a couple of parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Gravity", "gravity", g, 2.0, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame agelimit", "agelimit", p, 3, .5, 6.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 300.0, 1.0, 4000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Wind strength", "strength", wind, 0.0, 0.0, 10,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gustiness", "gustiness", wind, 3.0, 0.0, 20,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gust frequency", "gustfreq",
                            wind, 0.35, 0.1, 20,
                            VEE_MAP_LOGARITHMIC);


#############################################################################

# Make it visible:

print "Triangles"

print vg.triangleCount();
print vg.gridCount();
print vg.byteCount();


vq = VEE_Qt(scene)
# Make force invisible:
vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()

paramView.bangValues()

paramView.show()

print vq.renderer().subTextureCount()

qa.execute()

