# Simple fire effect

# Usual initializations:
from vee_init import *
from vee import *
import math
qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

agelimit = 2.0
gravity = 5.0

object = "tehdas3.lwo"
objectscale = 0.3;

objloc = VEE_Vector3();
objloc.make(0.0, -1.0, 0.0);

# Performance on pyry (rh 7.1)
# Points: 430
# lines: 430
# Quads (medium, stars): 340
# Quads (large): 250

# Complex (about 1200 particles per second) 75fps

# SPEEDS:
#
# RH 7.1: 320-370
#

#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_Particle()
p.thisown = 0

# p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(0.2, 0.2, 0.2, 0.0)
rgbatable.appendColor(color)
color.set(0.2, 0.2, 0.2, 1.0)
rgbatable.appendColor(color)
color.set(0.2, 0.2, 0.2, 1.0)
rgbatable.appendColor(color)
color.set(0.2, 0.2, 0.2, 1.0)
rgbatable.appendColor(color)
color.set(0.2, 0.2, 0.2, 0.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)


#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;
creator.setVisible(0)

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

range = 0.2

v1.make(-range, -range, -range)
v2.make( range,  range,  range)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())
vo.set("area", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)

#############################################################################

# Build a second particle with ugly colors, lifespan etc:

p2 = VEE_Particle()
p2.thisown = 0
p2.setAgeLimit(2);

# p.setAgeLimit(agelimit)

# rgbatable2 = VEE_RgbaTable()
# color2 = VEE_Rgba()

# color2.set(0.0, 0.0, 0.5, 1.0)
# rgbatable2.appendColor(color2)
#color2.set(0.2, 0.0, 0.7, 1.0)
#rgbatable2.appendColor(color2)
#color2.set(0.3, 0.0, 0.3, 1.0)
#rgbatable2.appendColor(color2)
#color2.set(0.0, 0.0, 0.0, 1.0)
#rgbatable2.appendColor(color2)

looks2 = VEE_LooksColorTable(rgbatable)
looks2.thisown = 0

valueLooks2 = VEE_ValueLooks()
valueLooks2.setLooks(looks2)

p2.setValue("looks", valueLooks2)



#############################################################################

# vortex = VEE_ObjectFactory_create("vortex")
# scene.addNewObject(vortex)
# vortex.setAgeLimit(2000.0)

#############################################################################

# Create a vortex generator:

vortex = VEE_ObjectFactory_create("vortex")
vortex.setAgeLimit(1.5);
vortex.setValueFloat("range", 1.0);
vortex.setValueFloat("magnitude", 0.5);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex)
screator.thisown = 0;
screator.setVisible(0)

# Random direction:
vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())
screator.setRate(1.0);

scene.addNewObject(screator)


#############################################################################

# Create second vortex generator:

vortex2 = VEE_ObjectFactory_create("vortex")
vortex2.setVelocityUpdate(1)

screator2 = VEE_Emitter(vortex2)
screator2.thisown = 0;
screator2.setVisible(0)

# Random direction:
vo.set("axes", rv.clone())
screator2.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator2.appendOperator(vo.clone())
screator2.setRate(1.0);

scene.addNewObject(screator2)


#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)
g.setValueInt("visible", 0);

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)

wind = VEE_ObjectFactory_create("wind")
scene.addNewObject(wind)

locationFollowers = VEE_ParamTargets()
locationFollowers.append(screator, "location");
locationFollowers.append(screator2, "location");

#grav = VEE_ObjectFactory_create("gravitate")
#grav.setValueFloat("range", 2.0);
#grav.setValueFloat("magnitude", -200); # Push away from object
#grav.setValueFloat("maxforce", 10);
#grav.setLocation3(1.0, 4.0, 0.0);
#scene.addNewObject(grav)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings fo a couple a few parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Gravity", "gravity", g, 0.5, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Smoke agelimit", "agelimit", p, 8, .5, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Smoke rate", "rate", creator, 38.0, 1.0, 4000.0,
                            VEE_MAP_LOGARITHMIC);

locEdit = paramView.addVector3Parameter("Smoke location", "location", creator,
                                        -1.9, -5.0, 5.0,
                                        1.8, -5.0, 5.0,
                                        0.3, -5.0, 5.0);

locEdit.addTargets(locationFollowers);

paramView.addFloatParameter("Vortex rate", "rate", screator, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex range", "range", vortex, 1.7, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex agelimit", "agelimit", vortex,
                            8.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, 0.19, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex2 rate", "rate", screator2, 3.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex2 range", "range", vortex2, 0.75, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex2 agelimit", "agelimit", vortex2,
                            8.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex2 magnitude", "magnitude", vortex2,
                            0.95, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.5, -2, 2,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind strength", "strength", wind, 3.0, 0.0, 10,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gustiness", "gustiness", wind, 1.0, 0.0, 20,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Wind gust frequency", "gustfreq",
                            wind, 0.35, 0.1, 20,
                            VEE_MAP_LOGARITHMIC);


#############################################################################

# Make it visible:

vq = VEE_Qt(scene)
q2 = VEE_TestQt(vq, scene, "", 640, 480, vq)
q2.setMtlFile("vee_tehdas.mtl");
vq.setGlWidget(q2);

vq.setFrameFile("frames/factory-%.6d.jpeg");

# Make force invisible:
vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()

vq.setValueFloat("GL/size", 1.1);
vq.setValueFloat("GL/alphamul", 0.2);
vq.setValueInt("GL/subtexture", 1);
vq.setValueRgbaf("GL/bgcolor", 0.6, 0.7, 0.8, 1.0);

vq.usual();

kp1 = vq.addKeyPoint2(0, 1.500, 10.000, 2.9, 0.000)
vq.addKeyPoint2(-5.590, -0.300, -7.151, 0.592, 0.000)
vq.selectKeyPoint2(kp1)

paramView.bangValues()

model = Solar_Model();

ok = Solar_Loaders_load(model, object);


model.centralize();

model.scale(objectscale);
solarobj = Solar_Object()

solarobj.setModel(model)
solarobj.setLocation(objloc)

q2.addModel(model);
q2.addObject(solarobj);

paramView.show()

print vq.renderer().subTextureCount()

qa.execute()

