
# Usual initializations:

from vee_init import *
from vee import *
import math

qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

agelimit = 5.0
gravity = 5.0

# object = "cat.lwo";
object = "town-flat.lwo";
# object = "saddle.lwo";
# object = "cupbase.lwo";
# object = "/usr/share/Performer/data/teapot.dxf";
objectscale = 0.3;

# objectLwo = object + ".lwo"
objectField = object + ".vfi"

model = Solar_Model();
model.centralize();
Solar_Loaders_load(model, object);

rebuildField = 0

if rebuildField:
    vgrid = VEE_VectorGrid();
    vgrid.createFromModel(model, 100, 3, 3, -1);
    vgrid.writeGridToFile(objectField)

    saveVfa = 0
    
    if saveVfa:
        vgrid.writeAsciiFile(object + ".vfa", vgrid.layers() / 2);

# res = VEE_VectorGrid_fromModelToFile(objectField, object, 70, 3, 3, 1);

rot = VEE_Matrix3();
rot.identity()

# rot.rotateY(-VEE_HALF_PI)
# rot.rotateZ(1.0)

#############################################################################

vgrid = VEE_VectorGrid();

res = vgrid.readGridFromFile(objectField)

gridforce = VEE_GridForce();

gridforce.setGrid(vgrid)
gridforce.setValueFloat("magnitude", 20.0);
gridforce.setValueFloat("scalemul", objectscale);
gridforce.setVisible(0);

objloc = VEE_Vector3();

objloc.make(0.0, 0.0, 0.0);

gridforce.setLocation(objloc);
gridforce.setValueMatrix3("rotation", rot);

scene.addNewObject(gridforce)

print "VISIBLE"
print gridforce.visible();

#############################################################################

# Build a particle with nice colors, lifespan etc:

p = VEE_Particle()
p.thisown = 0

# p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(1.0, 0.0, 0.0, 1.0)
rgbatable.appendColor(color)
color.set(0.9, 0.7, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.6, 0.6, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.0, 0.0, 0.0, 0.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)


#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

range = 0.2

v1.make(-range, -0.5 * range , -range)
v2.make( range,  0.5 * range,  range)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())
creator.setVisible(0)

scene.addNewObject(creator)

#############################################################################

# Build a second particle with ugly colors, lifespan etc:

p2 = VEE_Particle()
p2.thisown = 0
p2.setAgeLimit(2);

# p.setAgeLimit(agelimit)

# rgbatable2 = VEE_RgbaTable()
# color2 = VEE_Rgba()

# color2.set(0.0, 0.0, 0.5, 1.0)
# rgbatable2.appendColor(color2)
#color2.set(0.2, 0.0, 0.7, 1.0)
#rgbatable2.appendColor(color2)
#color2.set(0.3, 0.0, 0.3, 1.0)
#rgbatable2.appendColor(color2)
#color2.set(0.0, 0.0, 0.0, 1.0)
#rgbatable2.appendColor(color2)

looks2 = VEE_LooksColorTable(rgbatable)
looks2.thisown = 0

valueLooks2 = VEE_ValueLooks()
valueLooks2.setLooks(looks2)

p2.setValue("looks", valueLooks2)



#############################################################################

# vortex = VEE_ObjectFactory_create("vortex")
# scene.addNewObject(vortex)
# vortex.setAgeLimit(2000.0)

#############################################################################

# Create a vortex generator:

vortex = VEE_ObjectFactory_create("vortex")
vortex.setAgeLimit(1.5);
vortex.setValueFloat("range", 1.0);
vortex.setValueFloat("magnitude", 0.5);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex)
screator.thisown = 0;
screator.setVisible(0);

# Random direction:
vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())
screator.setRate(1.0);

scene.addNewObject(screator)


#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)
g.setValueInt("visible", 0);

repel = VEE_ObjectFactory_create("gravitate")
repel.setValueFloat("epsilon", 0.001);
scene.addNewObject(repel)
repel.setValueInt("visible", 0);

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)
damp.setValueInt("visible", 0);

locationFollowers = VEE_ParamTargets()
locationFollowers.append(repel, "location");

#grav = VEE_ObjectFactory_create("gravitate")
#grav.setValueFloat("range", 2.0);
#grav.setValueFloat("magnitude", -200); # Push away from object
#grav.setValueFloat("maxforce", 10);
#grav.setLocation3(1.0, 4.0, 0.0);
#scene.addNewObject(grav)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings fo a couple a few parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Gravity", "gravity", g, -1, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Gravitate decay", "decaypower", repel,
                            1.0, 0.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Gravitate magnitude", "magnitude", repel,
                            -10.0, -200 , 0.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Gravitate range", "range", repel,
                            1.0, .1 , 10.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Gravitate maxforce", "maxforce", repel,
                            10.0, .5 , 100.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Grid magnitude", "magnitude", gridforce,
                            60.0, .5, 1000.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Grid bounciness", "bounce", gridforce,
                            0.0, 0.0, 1.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid drag", "drag", gridforce,
                            0.0, 0.0, 7.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid drag power", "dragpower", gridforce,
                            2.0, 0.0, 3.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid tangent force", "tangentforce", gridforce,
                            9.0, -100.0, 100.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid tangent grab", "tangentgrab", gridforce,
                            0.9, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid threshold", "threshold", gridforce,
                            0.0, 0.0, 1.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame agelimit", "agelimit",
                            p, 1.7, .2, 4.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 50.0, 0.0, 3000.0,
                            VEE_MAP_LINEAR);

locEdit = paramView.addVector3Parameter("Flame location", "location", creator,
                                        0.49, -7.0, 7.0,
                                        -.84, -7.0, 7.0,
                                        -.2, -7.0, 7.0);

locEdit.addTargets(locationFollowers);

paramView.addFloatParameter("Vortex rate", "rate", screator, 0.0, 0.0, 3.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Vortex range", "range", vortex, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, 3.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex maxforce", "maxforce", vortex, 10.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.2, -2, 6,
                            VEE_MAP_LINEAR);

#############################################################################

# keypoint location = [0.467 0.300 9.545] azi = 3.142, ele = 0.000

# Make it visible:

vq = VEE_Qt(scene)
# Make force invisible:
vq.resize(500, 430);
q2 = VEE_TestQt(vq, scene, "", 640, 480, vq)
q2.setMtlFile("vee_materials.mtl");
vq.setGlWidget(q2);

vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()
# vq.setRenderOrder("1");
vq.setValueFloat("GL/size", 0.4);
vq.setValueFloat("GL/alphamul", 0.55);
vq.setValueInt("GL/subtexture", 1);
#vq.setValueInt("GL/renderorder", 2);
#vq.setValueInt("GL/blendtype", 2);

vq.addKeyPoint2(1.160, 0.525, -2.314, -0.558, -0.300);
vq.addKeyPoint2(1.744, 1.059, 0.441, 4.292, -0.800)
vq.addKeyPoint2(-0.807, 2.669, -1.030, 0.492, -1.100)
vq.addKeyPoint2(1.572, -0.741, 0.709, 4.142, 0.000)
vq.selectKeyPoint("3");

model = Solar_Model();

ok = Solar_Loaders_load(model, object);

model.centralize();
model.scale(objectscale);
solarobj = Solar_Object()

solarobj.setModel(model)
solarobj.setLocation(objloc)
solarobj.setRotation(rot)

q2.addModel(model);
q2.addObject(solarobj);

paramView.bangValues()

paramView.show()

qa.execute()

